/*
 * linux/arch/arm/mach-omap/board-h3.c
 *
 * This file contains OMAP1710 H3 specific code.
 *
 * Copyright (C) 2004 Texas Instruments, Inc.
 * Copyright (C) 2002 MontaVista Software, Inc.
 * Copyright (C) 2001 RidgeRun, Inc.
 * Author: RidgeRun, Inc.
 *         Greg Lonnon (glonnon@ridgerun.com) or info@ridgerun.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/delay.h>

#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>

#include <asm/arch/clocks.h>
#include <asm/arch/gpio.h>
#include <asm/arch/usb.h>

#include "common.h"

extern void __init omap_init_time(void);

void h3_init_irq(void)
{
	omap_init_irq();
}

static struct resource smc91x_resources[] = {
	[0] = {
		.start	= OMAP1710_ETHR_START,		/* Physical */
		.end	= OMAP1710_ETHR_START + OMAP1710_ETHR_SIZE,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= 0,
		.end	= 0,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device smc91x_device = {
	.name		= "smc91x",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(smc91x_resources),
	.resource	= smc91x_resources,
};
#ifdef VD1185T_R01
#define OMAP1710_DM9000_ADDR            0x06000000  //CS1B
#define OMAP1710_DM9000_DATA            0x06000002
 
static struct resource dm9000_resources[] = {
	[0] = {
		.start  = OMAP1710_DM9000_ADDR,          /* Physical */
		.end    = OMAP1710_DM9000_ADDR + 1,
		.flags  = IORESOURCE_MEM,
	},
	[1] = {
		.start  = OMAP1710_DM9000_DATA,      /* Physical */
		.end    = OMAP1710_DM9000_DATA + 1,
		.flags  = IORESOURCE_MEM,
	},
	[2] = {
		.start  = OMAP_GPIO_IRQ(25),
		.end    = 0,
		.flags  = IORESOURCE_IRQ,
	},
};
 
static struct platform_device dm9000_device = {
	.name           = "dm9000",
	.id             = 0,
	.num_resources  = ARRAY_SIZE(dm9000_resources),
	.resource       = dm9000_resources,
};
#endif

#define GPTIMER_BASE		0xFFFB1400
#define GPTIMER_REGS(x)		(0xFFFB1400 + (x * 0x800))
#define GPTIMER_REGS_SIZE	0x46

static struct resource intlat_resources[] = {
	[0] = {
		.start	= GPTIMER_REGS(0),		/* Physical */
		.end	= GPTIMER_REGS(0) + GPTIMER_REGS_SIZE,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= INT_1610_GPTIMER1,
		.end	= INT_1610_GPTIMER1,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device intlat_device = {
	.name		= "omap_intlat",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(intlat_resources),
	.resource	= intlat_resources,
};

static struct platform_device *h3_devices[] __initdata = {
        &smc91x_device,
#ifdef VD1185T_R01
	&dm9000_device,
#endif
	&intlat_device,
};

static struct omap_usb_config h3_usb_config __initdata = {
	/* usb1 has a Mini-AB port and external isp1301 transceiver */
#if	defined(CONFIG_USB_GADGET_OMAP) && !defined(CONFIG_ISP1301_OMAP)
	.register_dev	= 1,
	.pins[1]	= 6,
#else
	.otg		= 2,
	.pins[1]	= 3,
#endif

#ifdef	CONFIG_USB_GADGET_OMAP
	.hmc_mode	= 19,	// 0:host(off) 1:dev|otg 2:disabled
	// .hmc_mode	= 21,	// 0:host(off) 1:dev(loopback) 2:host(loopback)
#elif	defined(CONFIG_USB_OHCI_HCD) || defined(CONFIG_USB_OHCI_HCD_MODULE)
	/* NONSTANDARD CABLE NEEDED (B-to-Mini-B) */
	.hmc_mode	= 20,	// 1:dev|otg(off) 1:host 2:disabled
#endif
};

static struct omap_board_config_kernel h3_config[] = {
	{ OMAP_TAG_USB,		&h3_usb_config },
};


static void __init h3_init(void)
{
	platform_add_devices(h3_devices, ARRAY_SIZE(h3_devices));
	omap_board_config = h3_config;
	omap_board_config_size = ARRAY_SIZE(h3_config);
}

static struct map_desc h3_io_desc[] __initdata = {
{ OMAP1710_ETHR_BASE,  OMAP1710_ETHR_START,  OMAP1710_ETHR_SIZE,  MT_DEVICE },
};

static void __init h3_map_io(void)
{
	omap_map_io();
	iotable_init(h3_io_desc, ARRAY_SIZE(h3_io_desc));
}

MACHINE_START(OMAP_H3, "TI OMAP1710 H3 board")
	MAINTAINER("Texas Instruments, Inc.")
	BOOT_MEM(0x10000000, 0xfff00000, 0xfef00000)
	BOOT_PARAMS(0x10000100)
	MAPIO(h3_map_io)
	INITIRQ(h3_init_irq)
	INIT_MACHINE(h3_init)
	INITTIME(omap_init_time)
MACHINE_END
