/*
 * drivers/char/watchdog/omap_wdt_hardware.h
 *
 * Copyright (C) 2004 Texas Instruments Inc
 * Author:
 *
 * Hardware specific declarations used by Watchdog driver OMAP platforms. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

/*************Macros common to 16XX, 1710, & 24XX*************************/

#define OMAP_WATCHDOG_REV               (OMAP_WATCHDOG_BASE + 0x00)
#define OMAP_WATCHDOG_SYS_CONFIG        (OMAP_WATCHDOG_BASE + 0x10)
#define OMAP_WATCHDOG_STATUS            (OMAP_WATCHDOG_BASE + 0x14)
#define OMAP_WATCHDOG_CNTRL             (OMAP_WATCHDOG_BASE + 0x24)
#define OMAP_WATCHDOG_CRR               (OMAP_WATCHDOG_BASE + 0x28)
#define OMAP_WATCHDOG_LDR               (OMAP_WATCHDOG_BASE + 0x2c)
#define OMAP_WATCHDOG_TGR               (OMAP_WATCHDOG_BASE + 0x30)
#define OMAP_WATCHDOG_WPS               (OMAP_WATCHDOG_BASE + 0x34)
#define OMAP_WATCHDOG_SPR               (OMAP_WATCHDOG_BASE + 0x48)

/*********************Includes & Macros for 16XX and 1710 ********************************/

#if CONFIG_ARCH_OMAP16XX

#include <asm/arch/hardware.h>

#define OMAP1610_WATCHDOG_BASE  	0xfffeb000
#define OMAP_WATCHDOG_BASE 		OMAP1610_WATCHDOG_BASE

#define OPTIONS 			WDIOF_SETTIMEOUT
#define WDT_GET_BOOTSTATUS()    	(omap_readw(ARM_SYSST))

#define WDT_CK_ENABLE()			clk_enable(clk_info)
#define WDT_CK_DISABLE()  		clk_disable(clk_info)

#define TIMER_MARGIN_MAX        	32      /* Default is 32 seconds */
#define GET_WLDR_VAL(x)         	((0xffffffff - ((x) * 32768)) + 1)

/********************Macros for 24XX*******************************************/

#elif defined CONFIG_ARCH_OMAP24XX

#define OMAP_WATCHDOG_BASE 		OMAP24XX_VA_WDTIMER2_BASE

#define OPTIONS  			WDIOC_GETBOOTSTATUS | WDIOC_KEEPALIVE | 			\
					WDIOF_SETTIMEOUT| WDIOC_GETTIMEOUT

#define WDT_GET_BOOTSTATUS()    	(omap_prcmreg_read(PRCM_RM_RSTST_WKUP))

#define WDT_CK_ENABLE()			omap_prcmreg_write(omap_prcmreg_read(PRCM_CM_FCLKEN_WKUP)|(1<<3)\
					,PRCM_CM_FCLKEN_WKUP);						\
	    				omap_prcmreg_write(omap_prcmreg_read(PRCM_CM_ICLKEN_WKUP)|(1<<3)\
					,PRCM_CM_ICLKEN_WKUP)
	    

#define WDT_CK_DISABLE() 		omap_prcmreg_write(omap_prcmreg_read(PRCM_CM_FCLKEN_WKUP) &     \
					(~(1<<3)) ,PRCM_CM_FCLKEN_WKUP);				\
					omap_prcmreg_write(omap_prcmreg_read(PRCM_CM_ICLKEN_WKUP) & 	\
					(~(1<<3)) ,PRCM_CM_ICLKEN_WKUP)

#define GET_WLDR_VAL(x)   		((0xffffffff - ((x) * 32000)) + 1)

#define omap_prcmreg_read(reg) 		readl(OMAP24XX_VA_PRCM_BASE + reg)
#define omap_prcmreg_write(data,reg)	writel(data, OMAP24XX_VA_PRCM_BASE + reg)

#endif

