/*
 * linux/drivers/input/keyboard/omap-keypad.c
 *
 * OMAP Keypad Driver
 *
 * Copyright (C) 2003 Nokia Corporation
 * Written by Timo Ters <ext-timo.teras@nokia.com>
 *
 * Added support for H2, H3 Keypad
 * Copyright (C) 2004 Texas Instruments
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * History:
 *
 *                                     
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/types.h>
#include <linux/input.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <asm/arch/irqs.h>
#include <asm/arch/gpio.h>
#include <asm/arch/hardware.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <asm/mach-types.h>
#include <asm/arch/mux.h>
#include "omap-keypad.h"

#include <linux/kmsg.h>

#define QT_Select  0xffe0
#define QT_Back  0xffe1


static uint32_t t_PowerKeyPress;
static uint32_t t_PowerKeyRelease;

#define WIFI_PHONE_PM

#ifdef WIFI_PHONE_PM
//#define DEBUG_PM
#ifdef DEBUG_PM
#define info(format, arg...) 	PRINT_MSG(KMSG_MSG_LEVEL_INFOMATION, "", format...)
#define dbg(format, arg...)	PRINT_MSG(KMSG_MSG_LEVEL_DEBUG, "", format...)
#else
#define info(format, arg...)
#define dbg(format, arg...)
#endif



//#define DEBUG
#ifdef DEBUG
#define DPRINTK(ARGS...)  printk(KERN_INFO "<%s>: ",__FUNCTION__);printk(ARGS)
#define FN_IN printk(KERN_INFO "[%s]: start\n", __FUNCTION__)
#define FN_OUT(n) printk(KERN_INFO "[%s]: end(%u)\n",__FUNCTION__, n)
#else
#define DPRINTK( x... )
#define FN_IN
#define FN_OUT(x)
#endif


#include <linux/timer.h>
#include <asm/apm.h>
#include <linux/apm_bios.h>
static struct timer_list idle_timer;
static struct timer_list led_timer;

void omap_power_key_action(char key_press, uint32_t key_time);

static unsigned long idle_timeval = 0;
static int key_lock = 0;
static int idle_timer_active=0;
void set_keypad_backlight(int enable);
static int led_timer_active = 0;
void add_idle_timer(int timeval);
static void refresh_idle_timer(void);
static void refresh_lcm_timer(void);
static void add_keypad_backlight_timer(int interval);
static int kblight_timeval = 5;		//default 5 second
#define ADD_KBLIGHT_TIMER		add_keypad_backlight_timer(kblight_timeval)
static int pm_state;


#if defined(VM1185T_R01) || defined(VD1185T_R01)
#define KEYPAD_BACKLIGHT_GPIO  	34
#define R0C_MAP
#endif

#ifdef VM1185T_R0B
#define KEYPAD_BACKLIGHT_GPIO	50
#endif
#endif	//WIFI_PHONE_PM


//#define R0C_MAP

#ifdef R0C_MAP


static int h2_keymap[] = {

	//Select
	KEY(0, 0, 1),
	//Green(Call)
	KEY(1, 0, 2),
	// 1
	KEY(2, 0, 3),
	// 4
	KEY(3, 0, 4),
	// 7
	KEY(4, 0, 5),
	// *
	KEY(5, 0, 6),


	// 2
	KEY(2, 1, 7),
	// 5
	KEY(3, 1, 8),
	// 8
	KEY(4, 1, 9),
	// 0
	KEY(5, 1, 10),

	// Cancel
	KEY(0, 2, 11),

	//Red Key is not here

	// 3
	KEY(0, 3, 12),
	// 6
	KEY(1, 3, 13),
	// 9
	KEY(4, 3, 14),
	// #
	KEY(5, 3, 15),


	//Up
	KEY(1, 2, 16),
	//Right
	KEY(2, 2, 17),
	//Down
	KEY(3, 2, 18),
	//Left
	KEY(4, 2, 19),
	//Middle
	KEY(5, 2, 20),


	//Volume Up
	KEY(2, 3, 21),
	//Volume Down
	KEY(3, 3, 22),

	
	0
};



#else


static int h2_keymap[] = {

	//Select
	KEY(0, 0, 1),
	//Green(Call)
	KEY(1, 0, 2),
	// 1
	KEY(2, 0, 3),
	// 4
	KEY(3, 0, 4),
	// 7
	KEY(4, 0, 5),
	// *
	KEY(5, 0, 6),


	// 2
	KEY(2, 1, 7),
	// 5
	KEY(3, 1, 8),
	// 8
	KEY(4, 1, 9),
	// 0
	KEY(5, 1, 10),

	// Cancel
	KEY(0, 3, 11),

	//Red Key is not here

	// 3
	KEY(2, 2, 12),
	// 6
	KEY(3, 2, 13),
	// 9
	KEY(4, 2, 14),
	// #
	KEY(5, 2, 15),


	//Up
	KEY(1, 3, 16),
	//Right
	KEY(2, 3, 17),
	//Down
	KEY(3, 3, 18),
	//Left
	KEY(4, 3, 19),
	//Middle
	KEY(5, 3, 20),

//Old Keypad
	//Up
	KEY(0, 2, 16),
	//Right
	KEY(1, 2, 17),
	//Down
	KEY(0, 1, 18),
	//Left
	KEY(1, 1, 19),
//---------------	


	//Volume Up
	KEY(0, 2, 21),
	//Volume Down
	KEY(1, 2, 22),

	
	0
};

#endif


static int test_keymap[] = {
	KEY(0, 0, KEY_F4),
	KEY(1, 0, KEY_LEFT),
	KEY(2, 0, KEY_F1),
	KEY(0, 1, KEY_DOWN),
	KEY(1, 1, KEY_ENTER),
	KEY(2, 1, KEY_UP),
	KEY(0, 2, KEY_F3),
	KEY(1, 2, KEY_RIGHT),
	KEY(2, 2, KEY_F2),
	0
};

static int innovator_keymap[] = {
	KEY(0, 0, KEY_F1),
	KEY(0, 3, KEY_DOWN),
	KEY(1, 1, KEY_F2),
	KEY(1, 2, KEY_RIGHT),
	KEY(2, 0, KEY_F3),
	KEY(2, 1, KEY_F4),
	KEY(2, 2, KEY_UP),
	KEY(3, 2, KEY_ENTER),
	KEY(3, 3, KEY_LEFT),
	0
};

static int osk_keymap[] = {
	KEY(0, 0, KEY_F1),
	KEY(0, 3, KEY_UP),
	KEY(1, 1, KEY_LEFTCTRL),
	KEY(1, 2, KEY_LEFT),
	KEY(2, 0, KEY_SPACE),
	KEY(2, 1, KEY_ESC),
	KEY(2, 2, KEY_DOWN),
	KEY(3, 2, KEY_ENTER),
	KEY(3, 3, KEY_RIGHT),
	0 
}; 

static int *keymap;
static void omap_keyevent_tasklet(unsigned long data);
DECLARE_TASKLET(keyevent_tasklet, omap_keyevent_tasklet, 0);
static irqreturn_t omap_kp_interrupt(int irq, void *dev_id,
				     struct pt_regs *regs)
{

        omap_writew(1, OMAP_MPUIO_BASE + OMAP_MPUIO_KBD_MASKIT);
	tasklet_schedule(&kp_tasklet);
	tasklet_schedule(&keyevent_tasklet);

	return IRQ_HANDLED;
}

static void omap_keyevent_tasklet(unsigned long data)
{

#ifdef WIFI_PHONE_PM
	if(!key_lock) {
		set_keypad_backlight(1);	//turn on keypad backlight
		ADD_KBLIGHT_TIMER;		//add timer to turn off keypad backlight
		refresh_idle_timer();
	}
			
	//refresh_lcm_timer();
#endif //WIFI_PHONE_PM
}
static void omap_kp_timer(unsigned long data)
{
	tasklet_schedule(&kp_tasklet);
}
static void omap_kp_scan_keypad(unsigned char *state)
{
	int col = 0;
	//unsigned short *reg_info = (unsigned short *)state;

	/* read the keypad status */
	omap_writew(0xff, OMAP_MPUIO_BASE + OMAP_MPUIO_KBC);
	for (col = 0; col < 8; col++) {
	  omap_writew(~(1 << col) & 0x3f,
		      OMAP_MPUIO_BASE + OMAP_MPUIO_KBC);
	  
	  if (machine_is_omap_osk() || machine_is_omap_h2() || machine_is_omap_h3()) {
	    udelay(9);
	  } else {
	    udelay(2);
	  }

	  /*
	  state[col] =
	    ~omap_readw(OMAP_MPUIO_BASE +
			OMAP_MPUIO_KBR_LATCH) & 0x03;
	*/
	  state[col] =
	    ~omap_readw(OMAP_MPUIO_BASE +
			OMAP_MPUIO_KBR_LATCH) & 0xFF;


	//DPRINTK("Michael:omap_kp_scan_keypad:state[%d] = %d\n", col, state[col]);

	}

	omap_writew(0x00, OMAP_MPUIO_BASE + OMAP_MPUIO_KBC);
	udelay(2);
}
static inline int omap_kp_find_key(int col, int row)
{
	int i, key;

	key = KEY(col, row, 0);
	DPRINTK("Michael:omap_kp_find_key KEY(%d, %d, 0)=%d\n",col,row,key);
	for (i = 0; keymap[i] != 0; i++)
		if ((keymap[i] & 0xff000000) == key)
			return keymap[i] & 0x00ffffff;
	return -1;
}
static void omap_kp_tasklet(unsigned long data)
{


	unsigned char new_state[8], changed, key_down = 0;
	int col, row;
	int spurious = 0;

	
	/* check for any changes */
	omap_kp_scan_keypad(new_state);

	/* check for changes and print those */
	for (col = 0; col < 8; col++) {
		changed = new_state[col] ^ keypad_state[col];
		key_down |= new_state[col];
		for (row = 0; row < 8; row++) {
			int key;
			if ((!(changed & (1 << row)))
			    && (!(new_state[col] & (1 << row))))
				continue;
#ifdef NEW_BOARD_LEARNING_MODE
			DPRINTK(KERN_INFO "omap-keypad: key %d-%d %s\n", col,
			       row,
			       (new_state[col] & (1 << row)) ? "pressed" :
			       "released");
#else
			key = omap_kp_find_key(col, row);
			if ( key < 0 ) {
				DPRINTK(KERN_WARNING
				       "omap-keypad: Spurious key event %d-%d\n",
				       col, row);
				/* We scan again after a couple of seconds */
				spurious = 1;
				continue;
			}
			else
			{
				DPRINTK("Michael:omap_kp_tasklet::omap_kp_find_key:key = %d  (%d - %s)\n", key,  new_state[col] & (1 << row),(new_state[col] & (1 << row)) ? "pressed" :"released");
			}

//Michael's Backlight Test
/*
if (key==103)
{
			omap_set_gpio_direction(50, 0);
			omap_set_gpio_dataout(50, 1);

}
else if  (key==108)
{


			omap_set_gpio_direction(50, 0);
			omap_set_gpio_dataout(50, 0);
}


			if( (new_state[col] & (1 << row)) )
			{
*/
				//Press
				if(get_lcm_backlight_status())
					input_report_key(&omap_kp_dev, key, new_state[col] & (1 << row));

/*
			}
			else
			{
				//Release
				key |= 0x80;
				input_report_key(&omap_kp_dev, key, new_state[col] & (1 << row));
			}
*/
			DPRINTK("Michael:omap_kp_tasklet::omap_kp_find_key: Realkey = %d\n", key);


#endif
		}
	}
	memcpy(keypad_state, new_state, sizeof(keypad_state));

	if (key_down) {
		//int delay = HZ / 500;
		int delay = HZ / 100;

		/* some key is pressed - keep irq disabled and use timer
		 * to poll the keypad */
		if (spurious)
			delay = 2 * HZ;
		if(get_lcm_backlight_status())
			mod_timer(&kp_timer, jiffies + delay);
		else
			mod_timer(&kp_timer, jiffies + HZ/4);
		refresh_lcm_timer();
	} else {
		/* TODO: enable interrupts */
	        omap_writew(0, OMAP_MPUIO_BASE + OMAP_MPUIO_KBD_MASKIT);
	}
}

static int __init omap_kp_init(void)
{
	int i;

	FN_IN;
	/* Disable the interrupt for the MPUIO keyboard */
	omap_writew(1, OMAP_MPUIO_BASE + OMAP_MPUIO_KBD_MASKIT);

	if (machine_is_omap_h2() || machine_is_omap_h3()) {
		keymap = h2_keymap;
		set_bit(EV_REP, omap_kp_dev.evbit);
	} else if (machine_is_omap_innovator()) {
		keymap = innovator_keymap;
	} else if (machine_is_omap_osk()) {
		keymap = osk_keymap;
	} else {
		keymap = test_keymap;
	}

	init_timer(&kp_timer);
	kp_timer.function = omap_kp_timer;

	/* get the irq and init timer */
	tasklet_enable(&kp_tasklet);
	if (request_irq(INT_KEYBOARD, omap_kp_interrupt, 0,
			"omap-keypad", 0) < 0)
		return -EINVAL;

	/* setup input device */
	set_bit(EV_KEY, omap_kp_dev.evbit);
	for (i = 0; keymap[i] != 0; i++)
		set_bit(keymap[i] & 0x00ffffff, omap_kp_dev.keybit);
	omap_kp_dev.name = "omap-keypad";
	input_register_device(&omap_kp_dev);

	/* Write to CTL */
	if (machine_is_omap_h2() || machine_is_omap_h3()) {
		omap_cfg_reg(F18_1610_KBC0);
		omap_cfg_reg(D20_1610_KBC1);
		omap_cfg_reg(D19_1610_KBC2);
		omap_cfg_reg(E18_1610_KBC3);
		omap_cfg_reg(C21_1610_KBC4);
		omap_cfg_reg(G18_1610_KBR0);
		omap_cfg_reg(F19_1610_KBR1);
		omap_cfg_reg(H14_1610_KBR2);
		omap_cfg_reg(E20_1610_KBR3);
		omap_cfg_reg(E19_1610_KBR4);
		// :omap_cfg_reg(N19_1610_KBR5);
		//Michael's Backlight Test
		//omap_cfg_reg(M18_1710_GPIO50);
		omap_writew(0xff, OMAP_MPUIO_BASE + OMAP_MPUIO_GPIO_DEBOUNCING);
	}

	/* scan current status and enable interrupt */
	omap_kp_scan_keypad(keypad_state);
	omap_writew(0, OMAP_MPUIO_BASE + OMAP_MPUIO_KBD_MASKIT);
#ifdef WIFI_PHONE_PM
#if defined(VM1185T_R01) || defined(VD1185T_R01)
	omap_cfg_reg(A17_1710_GPIO34);
        printk("VM1185T_R01 and VD1185T_R01:keypad\n");
#endif
#ifdef VM1185T_R01
        printk("VM1185T_R0B:keypad\n");
	omap_cfg_reg(M18_1710_GPIO50);	//ron_lee
#endif
	omap_request_gpio(KEYPAD_BACKLIGHT_GPIO);
	omap_set_gpio_direction(KEYPAD_BACKLIGHT_GPIO, 0);	//set KEYPAD_BACKLIGHT_GPIO as output
#endif	//WIFI_PHONE_PM
	return 0;
}
static void __exit omap_kp_exit(void)
{
	/* disable keypad interrupt handling */
	tasklet_disable(&kp_tasklet);
	omap_writew(1, OMAP_MPUIO_BASE + OMAP_MPUIO_KBD_MASKIT);

	free_irq(INT_KEYBOARD, 0);
	del_timer_sync(&kp_timer);
#ifdef WIFI_PHONE_PM
	del_timer_sync(&idle_timer);	//delete idle timer
#endif
	/* unregister everything */
	input_unregister_device(&omap_kp_dev);
}

#ifdef WIFI_PHONE_PM
static void idle_monitor(unsigned long timeval)
{
	//timeval timeout, then send a event to APM
#if CONFIG_APM
	apm_queue_event(WIFI_EVENT_IDLE);	
#endif
	del_timer(&idle_timer);		//yes, delete the idle timer
	idle_timer_active = 0;
	idle_timeval = 0;
}
void add_idle_timer(int timeval)
{
	idle_timeval = timeval;
	if(idle_timeval == 0)
		return;
	if(idle_timer_active)
		//if timer already exist, then refresh the idle expire
		mod_timer(&idle_timer, jiffies + timeval * HZ);	
	else {
		init_timer(&idle_timer);
		idle_timer.function = idle_monitor;
		idle_timer.expires = jiffies + timeval*HZ;	
		idle_timer.data = (unsigned long) idle_timeval;
		add_timer(&idle_timer);		//add idle timer 
		idle_timer_active=1;		
	}
}
EXPORT_SYMBOL(add_idle_timer);
void del_idle_timer()
{
	idle_timeval = 0;
	if(idle_timer_active) {
		del_timer(&idle_timer);	//delete idle timer
		idle_timer_active = 0;
	}
}
EXPORT_SYMBOL(del_idle_timer);
int get_idle_timer()
{
	return idle_timeval;
}
EXPORT_SYMBOL(get_idle_timer);
static void refresh_idle_timer()
{
#if CONFIG_APM
	pm_state = apm_get_power_state();
#endif
	if(idle_timer_active)
		mod_timer(&idle_timer, jiffies + idle_timeval *HZ);
#if CONFIG_APM
	//if(pm_state == APM_STATE_IDLE_MODE || pm_state == APM_STATE_SUSPEND)
		apm_queue_event(WIFI_EVENT_ANY_KEY);
#endif
}
static void refresh_lcm_timer()
{
	set_lcd_backlight(1);
	add_lcmbklight_timer();
}
void set_keypad_backlight(int enable)
{
	omap_set_gpio_dataout(KEYPAD_BACKLIGHT_GPIO, enable);
}
EXPORT_SYMBOL(set_keypad_backlight);
static void backlight_timer(unsigned long data)
{
	set_keypad_backlight(0);	//turn off keypad backlight
	del_timer(&led_timer);		//remove the timer
	led_timer_active = 0;
}
static void add_keypad_backlight_timer(int interval)
{
	if(interval == 0)
		return;
	if(led_timer_active)
		mod_timer(&led_timer, jiffies + interval * HZ);
	else
	{	
		init_timer(&led_timer);
		led_timer.function = backlight_timer;
		led_timer.expires = jiffies + interval * HZ;
		led_timer.data = 0;
		add_timer(&led_timer);
		led_timer_active = 1;
	}
}
void add_kblight_timer()
{
	add_keypad_backlight_timer(kblight_timeval);
}
EXPORT_SYMBOL(add_kblight_timer);
void set_kblight_timeval(int timeval)
{
	kblight_timeval = timeval;
}
EXPORT_SYMBOL(set_kblight_timeval);

int get_kblight_timeval()
{
	return kblight_timeval;
}
EXPORT_SYMBOL(get_kblight_timeval);
#endif	//WIFI_PHONE_PM


//key_press=1 --> press key; key_release=0 --> release key
//key_time -> Use jiffies as time
void omap_power_key_action(char key_press, uint32_t key_time)
{

	int keycode;
	//press key
	if (key_press == 1)	//key pressed
	{
		t_PowerKeyPress = key_time;
#ifdef WIFI_PHONE_PM
		if(!key_lock) {
			set_keypad_backlight(1);
			ADD_KBLIGHT_TIMER;
		}
		refresh_idle_timer();
		refresh_lcm_timer();
#endif
	}
	//release key
	else if (key_press == 0)
	{
	//unsigned char new_state[8];
	/* check for any changes */
	//omap_kp_scan_keypad(new_state);		
	//Michael:Must modify /drivers/input/Input.c
		t_PowerKeyRelease = key_time - t_PowerKeyPress;

		DPRINTK("Michael:omap_kp_tasklet::Press = %lu, Release = %lu, During = %lu\n", t_PowerKeyPress, key_time, t_PowerKeyRelease);

		//Red Key
		if ( t_PowerKeyRelease < 100 )
		{
			//keycode = 23|0x80;
			keycode = 23;
			input_report_key(&omap_kp_dev, keycode, 1);
			DPRINTK("Michael:omap_kp_tasklet::Red Key\n");

		}
		//Power Menu
		else if ( t_PowerKeyRelease < 400 )
		{
			//keycode = 24|0x80;
			keycode = 24;
			input_report_key(&omap_kp_dev, keycode, 1);
			DPRINTK("Michael:omap_kp_tasklet::Power Menu\n");		

		}
		//Power Down
		else
		{
			//keycode = 25|0x80;
			keycode = 25;
			input_report_key(&omap_kp_dev, keycode, 1);
			DPRINTK("Michael:omap_kp_tasklet::Power Down\n");
		}

	}
	// Error
	else
	{
			DPRINTK("Michael:omap_kp_tasklet::Key_State Error = %d; %c\n", key_press, key_press);

	}
}




module_init(omap_kp_init);
module_exit(omap_kp_exit);
EXPORT_SYMBOL(omap_power_key_action);

MODULE_AUTHOR("Timo Ters");
MODULE_DESCRIPTION("OMAP Keypad Driver");
MODULE_LICENSE("GPL");
