/*
 * drivers/media/video/omap/sensor_ov7660.c
 *
 * Ov7660 Sensor driver for OMAP camera sensor interface
 * Modify from the code: sensor_ov9640.c
 *
 * Author: 
 *
 * Copyright (C) 2005 Arcadyan Inc.
 * Copyright (C) 2004 MontaVista Software, Inc.
 * Copyright (C) 2004 Texas Instruments.
 *
 * This file is licensed under the terms of the GNU General Public License 
 * version 2. This program is licensed "as is" without any warranty of any 
 * kind, whether express or implied.
 */
 
/*
 * drivers/media/video/omap/sensor_ov9640.c
 *
 * Ov9640 Sensor driver for OMAP camera sensor interface
 *
 * Author: Andy Lowe (source@mvista.com)
 *
 * Copyright (C) 2004 MontaVista Software, Inc.
 * Copyright (C) 2004 Texas Instruments.
 *
 * This file is licensed under the terms of the GNU General Public License 
 * version 2. This program is licensed "as is" without any warranty of any 
 * kind, whether express or implied.
 */

#ifndef OV7660_H
#define OV7660_H

/* The OV7660 I2C sensor chip has a fixed slave address of 0x30. */

/* Need to check - no information in datasheet */
#ifdef CONFIG_OMAP24XX_VIRTIO
#define OV7660_I2C_ADDR		0x42
#else
#define OV7660_I2C_ADDR		0x21
#endif
/* Need to check - no information in datasheet */

/* define register offsets for the OV7660 sensor chip */
#define OV7660_GAIN		0x00
#define OV7660_BLUE		0x01
#define OV7660_RED		0x02
#define OV7660_VREF		0x03
#define OV7660_COM1		0x04
#define OV7660_BAVE		0x05
#define OV7660_GEAVE		0x06
#define OV7660_RAVE		0x08
#define OV7660_COM2		0x09
#define OV7660_PID		0x0A
#define OV7660_VER		0x0B
#define OV7660_COM3		0x0C
#define OV7660_COM4		0x0D
#define OV7660_COM5		0x0E
#define OV7660_COM6		0x0F
#define OV7660_AECH		0x10
#define OV7660_CLKRC		0x11
#define OV7660_COM7		0x12
#define OV7660_COM8		0x13
#define OV7660_COM9		0x14
#define OV7660_COM10		0x15
#define OV7660_HSTRT		0x17
#define OV7660_HSTOP		0x18
#define OV7660_VSTRT		0x19
#define OV7660_VSTOP		0x1A
#define OV7660_PSHFT		0x1B
#define OV7660_MIDH		0x1C
#define OV7660_MIDL		0x1D
#define OV7660_MVFP		0x1E
#define OV7660_LAEC		0x1F
#define OV7660_BOS		0x20
#define OV7660_GBOS		0x21
#define OV7660_GROS		0x22
#define OV7660_ROS		0x23
#define OV7660_AEW		0x24
#define OV7660_AEB		0x25
#define OV7660_VPT		0x26
#define OV7660_BBIAS		0x27
#define OV7660_GBBIAS		0x28
#define OV7660_EXHCH		0x2A
#define OV7660_EXHCL		0x2B
#define OV7660_RBIAS		0x2C
#define OV7660_ADVFL		0x2D
#define OV7660_ADVFH		0x2E
#define OV7660_YAVE		0x2F
#define OV7660_HSYST		0x30
#define OV7660_HSYEN		0x31
#define OV7660_HREF		0x32
#define OV7660_CHLF		0x33
#define OV7660_ARBLM		0x34
#define OV7660_ADC		0x37
#define OV7660_ACOM		0x38
#define OV7660_OFON		0x39
#define OV7660_TSLB		0x3A
#define OV7660_COM11		0x3B
#define OV7660_COM12		0x3C
#define OV7660_COM13		0x3D
#define OV7660_COM14		0x3E
#define OV7660_EDGE		0x3F
#define OV7660_COM15		0x40
#define OV7660_COM16		0x41
#define OV7660_COM17		0x42
#define OV7660_MTX1		0x4F
#define OV7660_MTX2		0x50
#define OV7660_MTX3		0x51
#define OV7660_MTX4		0x52
#define OV7660_MTX5		0x53
#define OV7660_MTX6		0x54
#define OV7660_MTX7		0x55
#define OV7660_MTX8		0x56
#define OV7660_MTX9		0x57
#define OV7660_MTXS		0x58
#define OV7660_LCC1		0x62
#define OV7660_LCC2		0x63
#define OV7660_LCC3		0x64
#define OV7660_LCC4		0x65
#define OV7660_LCC5		0x66
#define OV7660_MANU		0x67
#define OV7660_MANV		0x68
#define OV7660_HV		0x69
#define OV7660_MBD		0x6A
#define OV7660_DBLV		0x6B
#define OV7660_GSP1		0x6C
#define OV7660_GSP2		0x6D
#define OV7660_GSP3		0x6E
#define OV7660_GSP4		0x6F
#define OV7660_GSP5		0x70
#define OV7660_GSP6		0x71
#define OV7660_GSP7		0x72
#define OV7660_GSP8		0x73
#define OV7660_GSP9		0x74
#define OV7660_GSP10		0x75
#define OV7660_GSP11		0x76
#define OV7660_GSP12		0x77
#define OV7660_GSP13		0x78
#define OV7660_GSP14		0x79
#define OV7660_GSP15		0x7A
#define OV7660_GSP16		0x7B
#define OV7660_GST1		0x7C
#define OV7660_GST2		0x7D
#define OV7660_GST3		0x7E
#define OV7660_GST4		0x7F
#define OV7660_GST5		0x80
#define OV7660_GST6		0x81
#define OV7660_GST7		0x82
#define OV7660_GST8		0x83
#define OV7660_GST9		0x84
#define OV7660_GST10		0x85
#define OV7660_GST11		0x86
#define OV7660_GST12		0x87
#define OV7660_GST13		0x88
#define OV7660_GST14		0x89
#define OV7660_GST15		0x8A

#define OV7660_NUM_REGS		(OV7660_GST15 + 1)

/* Need to check - no information in datasheet */
#define OV7660_PID_MAGIC		0x76	/* high byte of product ID number */
#define OV7660_VER_REV2		        0x48	/* low byte of product ID number */
#define OV7660_VER_REV3		        0x49	/* low byte of product ID number */
#define OV7660_MIDH_MAGIC		0x7F	/* high byte of mfg ID */
#define OV7660_MIDL_MAGIC		0xA2	/* low byte of mfg ID */
/* Need to check - no information in datasheet */

/* define a structure for ov7660 register initialization values */
struct ov7660_reg {
	unsigned char reg;
	unsigned char val;
};

enum image_size { QQCIF, QQVGA, QCIF, QVGA, CIF, VGA };
enum pixel_format { YUV, RGB565, RGB555 };
#define NUM_IMAGE_SIZES 6
#define NUM_PIXEL_FORMATS 3

struct capture_size {
	unsigned long width;
	unsigned long height;
};

/* Array of image sizes supported by OV7660.  These must be ordered from 
 * smallest image size to largest.
 */
const static struct capture_size ov7660_sizes[] = {
	{   88,  72 },	/* QQCIF */
	{  160, 120 },	/* QQVGA */
	{  176, 144 },	/* QCIF */
	{  320, 240 },	/* QVGA */
	{  352, 288 },	/* CIF */
	{  640, 480 },	/* VGA */
};

#endif /* ifndef OV7660_H */

