
/*
 * drivers/media/video/omapcamera_sensor_if.h
 *
 * Sensor interface to Video-for-Linux (Version 2) camera capture driver for 
 * the OMAP24xx camera controller.
 *
 * Author: Andy Lowe (source@mvista.com)
 *
 * Copyright (C) 2004 MontaVista Software, Inc.
 * Copyright (C) 2004 Texas Instruments.
 *
 * This file is licensed under the terms of the GNU General Public License 
 * version 2. This program is licensed "as is" without any warranty of any 
 * kind, whether express or implied.
 */

#ifndef CAMSENSOR_H
#define CAMSENSOR_H
struct camera_sensor {
	unsigned int version;
	int (*dectect) (void);  /* detect this camera */
	void (*init)(struct omap24xxcam_device *cam); /* initialization */
	void (* cleanup) (void);
	int (*power_on) (struct omap24xxcam_device *cam);
	int (*power_off)(struct omap24xxcam_device *cam);
	int(*enum_pixformat)(int, struct v4l2_fmtdesc *);
	void (*try_format) (struct v4l2_pix_format *);
	int (*set_format) (struct v4l2_pix_format *);
	void (*get_format) (struct v4l2_pix_format *);
	void (*timeperframe) (struct omap24xxcam_device *cam);
	int (*set_frame_period) (unsigned long *xclk, struct v4l2_fract *fper);
	void (*get_frame_period) (unsigned long *xclk, struct v4l2_fract *fper);
	int (*query_control) (struct v4l2_queryctrl *qc);
	int  (* get_control) (struct  v4l2_queryctrl *vc);
	int  (* set_control) (struct  v4l2_queryctrl *vc);
	int (*configure) (struct omap24xxcam_device *cam);
	void (*exit)(struct omap24xxcam_device *cam);
	int (*ioctl)(struct omap24xxcam_fh *fh, unsigned long cmd, void *arg);
};

struct ov9640_sensor {
	/* I2C parameters */
	struct i2c_client client;
	struct i2c_driver driver;

	int ver; /* OV9640 version */
};

	


enum image_size { QQCIF, QQVGA, QCIF, QVGA, CIF, VGA, SXGA };
enum pixel_format { YUV, RGB565, RGB555 };
#define NUM_IMAGE_SIZES 7
#define NUM_PIXEL_FORMATS 3

struct capture_size {
	unsigned long width;
	unsigned long height;
};

/* Array of image sizes supported by OV9640.  These must be ordered from 
 * smallest image size to largest.
 */
const static struct capture_size ov9640_sizes[] = {
	{   88,  72 },	/* QQCIF */
	{  160, 120 },	/* QQVGA */
	{  176, 144 },	/* QCIF */
	{  320, 240 },	/* QVGA */
	{  352, 288 },	/* CIF */
	{  640, 480 },	/* VGA */
	{ 1280, 960 },	/* SXGA */
};

#endif
