/*
 * kmfg.c -- the virtual fnull char module
 *           Directly read/write flash data(not from file system)
 *           To write code quickly, this code is 2.4 kernel style.
 *		There are two interface for kmfg.c, one is for DCL_MFG(application)
 *		, another is for other drivers.  The other drivers directly call
 * 		the API function(shall be extern).  The DCL_MFG uses the 
 *		open/read/write/ioctl to get/set flash data.   The other section
 * 		including flash checksum test and SDRAM test is maybe used for ESM.
 *		
 *		Author: Nick Wang, Accton Co,Ltd.
 *		Date: 2006/FEB/10
 */
#include <linux/module.h>

#include <linux/config.h>
//#include <linux/moduleparam.h>
#include <linux/init.h>

#include <linux/kernel.h>	/* printk() */
#include <linux/slab.h>		/* kmalloc() */
#include <linux/fs.h>		/* everything... */
#include <linux/errno.h>	/* error codes */
#include <linux/types.h>	/* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h>	/* O_ACCMODE */
#include <linux/seq_file.h>
#include <linux/cdev.h>

#include <asm/system.h>		/* cli(), *_flags */
#include <asm/uaccess.h>	/* copy_*_user */
#include <asm/io.h>
#include <asm/arch/hardware.h>
#include <asm/kmfg.h>

#include "kmfg_ioctl.h"		/* local definitions */

/*
 * Our parameters which can be set at load time.
 */

int scull_major =   SCULL_MAJOR;
int scull_minor =   0;
unsigned long flash_start_addr = 0x0; //debby added
unsigned int flash_size = 0;

struct scull_dev *scull_devices;	/* allocated in scull_init_module */

static int omap_flash_write(loff_t to, size_t len, const u_char * buf);
static int omap_flash_write_word(__u32 offset, __u16 x);
static int omap_flash_read(int len, u_char * buf);

static int omap_flash_erase(__u32 addr);
static inline int omap_flash_sector_erase(__u32 offset);


//omap_nor_flash_base is from omap-nor-flash.c
extern unsigned long omap_nor_flash_base;
//extern unsigned E_UBOOT_DATA[8];

#define omap_flash_inw(o) readw((omap_nor_flash_base+(__u32)o))
#define omap_flash_outw(o,d) writew((__u16)(d),(omap_nor_flash_base+(__u32)o))
#define BUSWIDTH               2	/*warning: this is used only in some functions, 
the code is specific to 16-bit data bus */

#define OLD_MAC 6
#define INDICATOR 2
#define SIGNATURE1 6
#define SIGNATURE2 6
#define RESERVD 40
#define MFG_BLOCK_VERSION 4

#define MFG_BLOCK_VERSION_NO 0x00000100


#define INDICATOR_OFFSET OLD_MAC
#define SIGNATURE1_OFFSET OLD_MAC+INDICATOR
#define SIGNATURE2_OFFSET OLD_MAC+INDICATOR+SIGNATURE1
#define RESERVD_OFFSET OLD_MAC+INDICATOR+SIGNATURE1+SIGNATURE2
#define MFG_BLOCK_VERSION_OFFSET OLD_MAC+INDICATOR+SIGNATURE1 \
									+SIGNATURE2+RESERVD


#define BLOCK_OFFSET OLD_MAC+INDICATOR \
					+SIGNATURE1+SIGNATURE2 \
					+RESERVD+MFG_BLOCK_VERSION


#define PRE_LENGTH 	BLOCK_OFFSET

//old mac addr + mfg block version
//#define BLOCK_OFFSET 8+4

#define OLD_MAC_ADDR HW_POS


extern struct semaphore omap_flash_lock;  //come from kernel

/* kernel API function declaration and data type */
#define    KMFG_BLOCK_ID_ALL			10  //it is added to gather all of data

unsigned char do_checksum(char *buffer, int buf_len, unsigned long *chksum);
int verify_checksum(char *buffer, int buf_len);

void init_mfg_block(void);

int system_len = (sizeof(KMFG_SYSTEM_BLK_T)+10);
int contrast_len = (sizeof(KMFG_CONTRAST_BLK_T)+10);
int brightness_len = (sizeof(KMFG_BRIGHTNESS_BLK_T)+10);
int battery_len = (sizeof(KMFG_BATTERY_BLK_T)+10);
int volume_len = (sizeof(KMFG_VOLUME_BLK_T)+10);
int wifi_len = (sizeof(KMFG_WIFI_BLK_T)+10);
int tot_len = 0;
unsigned int set_block_id = 0;

/*
	API for Other driver
*/
/**   Name:     kmfg_get_data
 **   Function: driver retrieve the information saved in mfg-info block.
 **   Input:
 **   @param    block_id - the identifier of block
 **   Output:
 **   @param    buffer - buffer for retrieved data not including signature,checksum,len
 **   @param    buf_len - the buffer for receving not including signature,checksum,len
 **   @retval   > 0 - the length of data returned.
 **   @retval   DCL_ERR_MFG_BLOCK_ID_UNDEFINED - invalid block ID.
 **   @retval   DCL_ERR_MFG_BUFFER_TOO_SMALL - the specified buffer is too small for the block information
 **   @retval   DCL_ERR_MFG_CHECKSUM - the block information is corrupt
 **   Notes:
 **     1. If buffer is NULL, buf_len return the buffer size caller should be prepared.
 **/
 EXPORT_SYMBOL(kmfg_get_data);
int kmfg_get_data(KMFG_BLOCK_ID_T block_id, char *buffer, int *buf_len)
{
	unsigned long tmp_flash_start_addr = 0;
	unsigned long chksum = 0;
	int t_buf_len = 0;
	char tbuffer[2000];

	switch(block_id)
	{
	  case KMFG_BLOCK_ID_SYSTEM:
		 if (*buf_len < (system_len-10)) return DCL_ERR_MFG_BUFFER_TOO_SMALL;
		 t_buf_len = system_len;
		 tmp_flash_start_addr = BLOCK_OFFSET;
		 break;
	  case KMFG_BLOCK_ID_CONTRAST:
		 if (*buf_len < (contrast_len-10)) return DCL_ERR_MFG_BUFFER_TOO_SMALL;
		 t_buf_len = contrast_len;
		 tmp_flash_start_addr = (BLOCK_OFFSET + system_len);
		 break;
	  case KMFG_BLOCK_ID_BRIGHTNESS:
		 if (*buf_len < (brightness_len-10)) return DCL_ERR_MFG_BUFFER_TOO_SMALL;
		 t_buf_len = brightness_len;
		 tmp_flash_start_addr = (BLOCK_OFFSET + system_len + contrast_len);
		 break;
	  case KMFG_BLOCK_ID_BATTERY:
		 if (*buf_len < (battery_len-10)) return DCL_ERR_MFG_BUFFER_TOO_SMALL;
		 t_buf_len = battery_len;
		 tmp_flash_start_addr = (BLOCK_OFFSET + system_len + contrast_len + brightness_len);
		 break;
	  //case KMFG_BLOCK_ID_VOLUME:
	  case KMFG_BLOCK_ID_HND_VOLUME:
	  case KMFG_BLOCK_ID_HED_VOLUME:
	  case KMFG_BLOCK_ID_HND_GAIN:
	  case KMFG_BLOCK_ID_HED_GAIN:
		 if (*buf_len < (volume_len-10)) return DCL_ERR_MFG_BUFFER_TOO_SMALL;
		 t_buf_len = volume_len;
		 tmp_flash_start_addr = (BLOCK_OFFSET + system_len + contrast_len + brightness_len + battery_len);
		 break;
	  case KMFG_BLOCK_ID_WIFI:
		 if (*buf_len < (wifi_len-10)) return DCL_ERR_MFG_BUFFER_TOO_SMALL;
		 t_buf_len = wifi_len;
		 tmp_flash_start_addr = (BLOCK_OFFSET + system_len + contrast_len + brightness_len + battery_len+ volume_len);
		 break;
	  case KMFG_BLOCK_ID_ALL:  //except
		 if (*buf_len < tot_len) return DCL_ERR_MFG_BUFFER_TOO_SMALL;
		 t_buf_len = tot_len;
		 tmp_flash_start_addr = BLOCK_OFFSET;
		 break;
	  default:
		 *buf_len = 0;
		 return DCL_ERR_MFG_BLOCK_ID_UNDEFINED;
	}

	/* if buffer pointer is NULL, return needed length, stupid idea */
	if (buffer == NULL)
	{
		return (t_buf_len-10);
	}

	/* get data from flash device, unit is Block */
	/* first read mfg block version to goto requested way */
	/* we need change flash_start_addr */
	//char* mfg_block_data = kmalloc(MFG_BLOCK_VERSION, GFP_KERNEL);
	int mfg_block_data;
	flash_start_addr = HW_POS + MFG_BLOCK_VERSION_OFFSET;
	omap_flash_read(MFG_BLOCK_VERSION, (u_char *) &mfg_block_data);
	
	/* if (mfg block version) != MFG_BLOCK_VERSION_NO, the mfg info is undefined, return ERROR */

	if (mfg_block_data != MFG_BLOCK_VERSION_NO)
	{
		printk("MFG_BLOCK_VERSION_NO=0x%x  Error!\n", mfg_block_data);
		//kfree(mfg_block_data);
		return DCL_ERR_MFG_BLOCK_ID_UNDEFINED;
	}
	//kfree(mfg_block_data);
	printk("MFG_BLOCK_VERSION_NO=0x%x  O.K.!\n", mfg_block_data);
	
	
	//read data from block beginning
	flash_start_addr = HW_POS + tmp_flash_start_addr;
	printk("[kmfg_get_data]block_id=%d, flash_start_addr=0x%x\n", block_id, flash_start_addr);

	/* get the all of data on block and do the checksum(byte + byte...) */
	omap_flash_read(t_buf_len, (u_char *) buffer);



	int chk = verify_checksum(buffer, t_buf_len);
	/* compare the checksum, if not equal, data is courrpted */
	if (do_checksum(buffer, t_buf_len, &chksum)) return DCL_ERR_MFG_CHECKSUM_ERROR;

	//for kernel device driver API, we substract the first 10 byte
	memcpy(tbuffer, buffer, t_buf_len);
	memcpy(buffer, tbuffer+10, t_buf_len-10);
	return (t_buf_len-10);
}


//Calculate checksum, from data to data
unsigned char do_checksum(char *buffer, int buf_len, unsigned long *chksum)
{
	unsigned int i,tot = 0;
	unsigned char *p = buffer;
	unsigned long *q;

	for(i=10;i<buf_len;i++)
	{
		//if (i<30) printk("%x ", *(p+i));
		tot += *(p+i);
	}

	q = (buffer + 4);
	*chksum = tot;
	if ((tot == *q) && (tot != 0)) 
	   return 0;
	else
	{
	   printk("\n***do_checksum error!::block_id=%d, tot=%d, *q=%d***\n", set_block_id, tot, *q);
	   return 1;
}
}


int verify_checksum(char *buffer, int buf_len)
{

	unsigned int i,j=0;
	unsigned char *p = buffer;
	

	//only first 4 bytes
	for(i=0;i<4;i++)
	{
		j += *p++;
	}

	return j;
}


EXPORT_SYMBOL(init_mfg_block);
void init_mfg_block()
{
	short ind = 0x0100;
	u_char *all_data;
	int i, retval;

/*
	char *sig1 = "ACCTON";
	char *sig2 = "ACCTON";
*/
	char *sig = "ACCTON";
	char *sig1 = kmalloc(SIGNATURE1+1, GFP_KERNEL);
	char *sig2 = kmalloc(SIGNATURE2+1, GFP_KERNEL);	

	char *reserved = kmalloc(RESERVD, GFP_KERNEL);
	int mfg_block_data = MFG_BLOCK_VERSION_NO;


	strcpy(sig1, sig);
	strcpy(sig2, sig);



	//printk("sig1\n");

	/*
	for(i=0;i<SIGNATURE1;i++)
		printk("0x%x ", *sig1++);

	sig1-= SIGNATURE1;
	
	printk("\n***%s\n", sig1);
	*/

	//Add sig1 code distance
	for(i=0;i<SIGNATURE1;i++)
	{

		*sig1 += i;
		//printk("0x%x ", *sig1);
		sig1++;
	}
	sig1 -= SIGNATURE1;


	//printk("\nsig2\n");


	//Add sig2 code distance
	for(i=0;i<SIGNATURE2;i++)
	{

		*sig2 += i;
		//printk("0x%x ", *sig2);
		sig2++;
	}
	sig2 -= SIGNATURE2;

//	printk("\n***%s\n", sig2);


/*
	printk("\nsig2 -> sig1\n");

	for(i=0;i<SIGNATURE2;i++)
	{

		*sig2 -= i;
		printk("0x%x ", *sig2);
		sig2++;
	}

	sig2-= SIGNATURE2;
	printk("\n***%s\n", sig2);
*/


//	printk("\nBefore Encrypt\n");
//	printk("\n***sig1=%s sig2=%s\n", sig1, sig2);

//	printk("\nEncrypt\n");
//	verify_checksum(sig1 -10, SIGNATURE1+10);
//	int chk = verify_checksum(sig2 -10, SIGNATURE2+10);
//	printk("\n***sig1=%s sig2=%s\n", sig1, sig2);

/*
	printk("\nDecrypt\n");
	verify_checksum(sig1 -10, SIGNATURE1+10);
	verify_checksum(sig2 -10, SIGNATURE2+10);
	printk("\n***sig1=%s sig2=%s\n", sig1, sig2);


	printk("\nEncrypt\n");
	verify_checksum(sig1 -10, SIGNATURE1+10);
	verify_checksum(sig2 -10, SIGNATURE2+10);
	printk("\n***sig1=%s sig2=%s\n", sig1, sig2);
*/

//	printk("\nRESERVD\n");
	for(i=0;i<RESERVD;i++)
	{
		*reserved = i;
		//printk("0x%x  ",*reserved);
		reserved++;
	}
	reserved -= RESERVD;

//	printk("\nRESERVD Encrypt\n");
	int chk =verify_checksum(reserved -10, RESERVD+10);

/*
	for(i=0;i<RESERVD;i++)
	{
		printk("0x%x  ",*reserved);
		reserved++;
	}
	reserved -= RESERVD;
*/
//--

	//Oh, my God, I need to save all of block data, because erase activity
	//will delete all of data forever
	all_data = kmalloc(tot_len, GFP_KERNEL);
	if (!all_data) {
	   printk("\n\n***init_mfg_block: memory allocation failed\n");
	   //return DCL_ERR_MFG_BLOCK_WRITE_ERROR;
	}
	flash_start_addr = HW_POS;
	omap_flash_read(tot_len, all_data);
	msleep(1);
	/*
	printk("\n\n<1>***");
	for(i=0 ;i < BLOCK_OFFSET;i++)
	{
		if( (i==INDICATOR_OFFSET) || (i==SIGNATURE1_OFFSET) \
			 || (i==SIGNATURE2_OFFSET)  || (i==RESERVD_OFFSET))
		{
			printk("\n-----\n");
		}
		printk("0x%x \n",*(all_data+i));
	}
	*/

	//erase the flash 2nd block address:0x10000
	retval = omap_flash_erase(HW_POS);
	if(retval)
		printk("\n\n***omap_flash_erase failed!\n");

	//msleep(10);

	//Write data to indicator
	memcpy(all_data+INDICATOR_OFFSET, &ind, INDICATOR);


	//Write sig1
	memcpy(all_data+SIGNATURE1_OFFSET, sig1, SIGNATURE1);

	//Write sig2
	memcpy(all_data+SIGNATURE2_OFFSET, sig2, SIGNATURE2);

	//Write reserved
	memcpy(all_data+RESERVD_OFFSET, reserved, RESERVD);

	//Write reserved
	memcpy(all_data+MFG_BLOCK_VERSION_OFFSET, &mfg_block_data, MFG_BLOCK_VERSION);

	flash_start_addr = HW_POS;
	retval = omap_flash_write(flash_start_addr, tot_len, (u_char *) all_data);

/*	
	if (retval < 0)
	{
		printk("\n\n***init_mfg_block: omap_flash_write failed! flash_start_addr=0x%x, tot_len=%d\n", flash_start_addr, tot_len);
		for(i=INDICATOR_OFFSET ;i < SIGNATURE1_OFFSET;i++)
			printk("\n\n***%x\n",*(all_data+i));
	}
	else
	{
		printk("\n\n***init_mfg_block: omap_flash_write O.k.!\n");
		for(i=INDICATOR_OFFSET ;i < SIGNATURE1_OFFSET;i++)
			printk("\n\n***%x\n",*(all_data+i));
	}
*/


	omap_flash_read(tot_len, all_data);
	printk("\n\n<2>***\n");
	for(i=0 ;i < BLOCK_OFFSET;i++)
	{
		if( (i==INDICATOR_OFFSET) || (i==SIGNATURE1_OFFSET) \
			 || (i==SIGNATURE2_OFFSET)  || (i==RESERVD_OFFSET))
		{
			printk("\n-----\n");
		}
		printk("0x%x \n",*(all_data+i));
	}


/*
	if(DCL_ERR_MFG < DCL_ERR_MFG_BLOCK_WRITE_ERROR)
	{
		printk("DCL_ERR_MFG < DCL_ERR_MFG_BLOCK_WRITE_ERROR\n");
	}
	else
	{
		printk("DCL_ERR_MFG > DCL_ERR_MFG_BLOCK_WRITE_ERROR\n");
	}	
*/	

	kfree(all_data);
	kfree(sig1);
	kfree(sig2);
	kfree(reserved);

//--
	

}

/*
 * Open and close
 */

int scull_open(struct inode *inode, struct file *filp)
{
	try_module_get(THIS_MODULE);
	return 0;          /* success */
}

int scull_release(struct inode *inode, struct file *filp)
{
	module_put(THIS_MODULE);
	return 0;
}


/*
 * Data management: read and write
 */
/* Get data from flas device including ljump and hardware information */
ssize_t scull_read(struct file *filp, char __user *buf, size_t count,
                loff_t *f_pos)
{
	ssize_t retval = 0;
	//char *p;
	size_t t_count = 0;

	//It is ugly to do more jobs over here, but I have no choice
	if (buf == NULL)
	{
		//get the required data length
		switch(set_block_id)
		{
	  	case KMFG_BLOCK_ID_SYSTEM:
			 return system_len;
			 break;
		case KMFG_BLOCK_ID_CONTRAST:
			 return contrast_len;
			 break;
		case KMFG_BLOCK_ID_BRIGHTNESS:
			 return brightness_len;
			 break;
		case KMFG_BLOCK_ID_BATTERY:
			 return battery_len;
			 break;
	  	//case KMFG_BLOCK_ID_VOLUME:
	  	case KMFG_BLOCK_ID_HND_VOLUME:
	  	case KMFG_BLOCK_ID_HED_VOLUME:
	  	case KMFG_BLOCK_ID_HND_GAIN:
	  	case KMFG_BLOCK_ID_HED_GAIN:
			 return volume_len;
			 break;
	  	case KMFG_BLOCK_ID_WIFI:
			 return wifi_len;
			 break;
	  	default:
			 return DCL_ERR_MFG_BLOCK_ID_UNDEFINED;
		}
	}
	
	t_count = count;
	//if ((fdata = kmalloc(t_count, GFP_KERNEL)) == NULL) 
	//	return DCL_ERR_MFG_BLOCK_READ_ERROR;
	retval = kmfg_get_data(set_block_id, fdata, &t_count);
	if ((retval & DCL_ERR_MFG) == 0)
		//copy_to_user(buf, fdata+10, t_count);
		copy_to_user(buf, fdata, t_count);
	//kfree(fdata);

	return retval;
}

/* DCL pass the data not including signature, checksum, and length, therefore */
/* the count is small than real block length(offset is 10).                   */
ssize_t scull_write(struct file *filp, const char __user *buf, size_t count,
                loff_t *f_pos)
{
	int retval = 0;
	unsigned long chksum = 0;
	char p0, p1, p2, p3;
	short t_count;
	short *q1;
	unsigned long *q2;
	unsigned long t_flash_start_addr;
	char *all_data;

     
	//allocate memory size of the needed block id
	switch(set_block_id)
	{
	  case KMFG_BLOCK_ID_SYSTEM:
		 if (count > (system_len-10)) return DCL_ERR_MFG_OVER_BLOCK_SIZE;
		 t_count = system_len;
		 p0='S';p1='Y';p2='S';p3='1';
		 t_flash_start_addr = HW_POS + BLOCK_OFFSET;
		 break;
	  case KMFG_BLOCK_ID_CONTRAST:
		 if (count > (contrast_len-10)) return DCL_ERR_MFG_OVER_BLOCK_SIZE;
		 t_count = contrast_len;
		 p0='C';p1='R';p2='S';p3='T';
		 t_flash_start_addr = HW_POS + BLOCK_OFFSET + system_len;
		 break;
	  case KMFG_BLOCK_ID_BRIGHTNESS:
		 if (count > (brightness_len-10)) return DCL_ERR_MFG_OVER_BLOCK_SIZE;
		 t_count = brightness_len;
		 p0='B';p1='R';p2='I';p3='T';
		 t_flash_start_addr = HW_POS + BLOCK_OFFSET + system_len + contrast_len;
		 break;
	  case KMFG_BLOCK_ID_BATTERY:
		 if (count > (battery_len-10)) return DCL_ERR_MFG_OVER_BLOCK_SIZE;
		 t_count = battery_len;
		 p0='B';p1='A';p2='T';p3='T';
		 t_flash_start_addr = HW_POS + BLOCK_OFFSET + system_len + contrast_len + brightness_len;
		 break;
	  //case KMFG_BLOCK_ID_VOLUME:
  	  case KMFG_BLOCK_ID_HND_VOLUME:
	  case KMFG_BLOCK_ID_HED_VOLUME:
	  case KMFG_BLOCK_ID_HND_GAIN:
	  case KMFG_BLOCK_ID_HED_GAIN:
		 if (count > (volume_len-10)) return DCL_ERR_MFG_OVER_BLOCK_SIZE;
		 t_count = volume_len;
		 p0='V';p1='O';p2='L';p3='M';
		 t_flash_start_addr = HW_POS + BLOCK_OFFSET + system_len + contrast_len + brightness_len + battery_len;
		 break;
	  case KMFG_BLOCK_ID_WIFI:
		 if (count > (wifi_len-10)) 
		 {
		     
		     printk("kevin 1 => wifi_len :%d ,count:%d  <==\n\r",wifi_len,count);
		 	return DCL_ERR_MFG_OVER_BLOCK_SIZE;
		 }
		 printk(" Enter wifi block \n\r");
		 t_count = wifi_len;
		 p0='W';p1='i';p2='F';p3='i';
		 //kevin modified : 20060429 
		 t_flash_start_addr = HW_POS + BLOCK_OFFSET + system_len + contrast_len + brightness_len + battery_len+volume_len;
		 break;
	  default:
		 t_count = 0;
		 return DCL_ERR_MFG_BLOCK_ID_UNDEFINED;
	}

	//Oh, my God, I need to save all of block data, because erase activity
	//will delete all of data forever
	all_data = kmalloc(tot_len, GFP_KERNEL);
	if (!all_data) {
	   printk("flash_agent: memory allocation failed\n");
	   return DCL_ERR_MFG_BLOCK_WRITE_ERROR;
	}
	flash_start_addr = HW_POS;
	omap_flash_read(tot_len, all_data);

	//erase the flash 2nd block address:0x10000
	omap_flash_erase(HW_POS);

/*
	fdata = kmalloc(t_count, GFP_KERNEL);
	if (!fdata) {
	   printk("flash_agent: memory allocation failed\n");
	   return DCL_ERR_MFG_BLOCK_WRITE_ERROR;
	}
*/
	//get data from DCL layer, and put to fdata offset 10, 
	//the begining is used for signature, checksum, and length
	if (copy_from_user(fdata+10, buf, count)) {
		
		 return DCL_ERR_MFG_BLOCK_WRITE_ERROR;
	}
       //kevin add debug
       printk("wifi_len:%d , count :%d\n\r",wifi_len,count);
   
	//First, calculate checksum, then write anything,
	//Then add signature, checksum, and block length
	//calculate checksum
	do_checksum(fdata, t_count, &chksum);
	*fdata = p0;*(fdata+1) = p1;*(fdata+2) = p2;*(fdata+3) = p3;
	q2 = fdata+4;*q2 = chksum;
	q1 = fdata+8;*q1 = t_count;

	int chk = verify_checksum(fdata, t_count);

	//prepare all of block data
	memcpy(all_data+t_flash_start_addr-HW_POS, fdata, t_count);

	flash_start_addr = HW_POS;
	retval = omap_flash_write(flash_start_addr, tot_len, (u_char *) all_data);
//	if (retval < DCL_ERR_MFG) retval = t_count - 10;

	if (retval < 0) retval = t_count - 10;
//	kfree(fdata);
	kfree(all_data);
	return retval;
}


/*
 *  flash operation
 */
//Nick, base from omap-nor-flash.c
/* Read a block of data from flash */
static int
omap_flash_read(int len, u_char * buf)
{
	/* We always read len bytes. */
	down(&omap_flash_lock);
	memcpy(buf, (void *) (omap_nor_flash_base + flash_start_addr), len);
	up(&omap_flash_lock);
	
	return (0);
}

//Just for compatibility
EXPORT_SYMBOL(omap_flash_read_mac);
int omap_flash_read_mac(u8* buf)
{
	/* We always read len bytes. */
	down(&omap_flash_lock);
	memcpy(buf, (void *) (omap_nor_flash_base + OLD_MAC_ADDR), 6);
	up(&omap_flash_lock);
	
	return (0);
}

/* Pick the proper sector addresses for erasing */
/* The HW information is on sector number 2 */
static int
omap_flash_erase(__u32 addr)
{
	/* now erase those blocks */
	while (1) {
		if (omap_flash_sector_erase(addr)) {
			return (-EIO);
		}
		else
		    break;
	}

	printk("\n\n***omap_flash_erase ok!\n");
	return (0);
}


/* Erase a flash sector */
static inline int
omap_flash_sector_erase(__u32 offset)
{
	__u16 status;

	down(&omap_flash_lock);
	/*erase sector */
	omap_flash_outw(offset, 0x0050);	/* clear status register */
	omap_flash_outw(offset, 0x0020);	/* erase setup */
	omap_flash_outw(offset, 0x00D0);	/* erase confirm */

	/* wait for erase complete while polling the status register */
	while (((status = omap_flash_inw(offset)) & 0x0080) != 0x0080)
		schedule_timeout(1);

	omap_flash_outw(offset, 0x0050);	/* clear status register */
	omap_flash_outw(offset, 0x00FF);	/* reset to read mode    */
	up(&omap_flash_lock);

	/* process status register */
	if (status & (1 << 3)) {
		printk("ERROR: erasing at address 0x%.8x: Vpp range error \n", offset);
		return -EPERM;
	}
	if ((status & (3 << 4)) == (3 << 4)) {
		printk("ERROR: erasing at address 0x%.8x: command sequence error\n", offset);
		return -EPERM;
	}
	if (status & (1 << 5)) {
		printk("ERROR: erasing at address 0x%.8x: block erase error\n", offset);
		return -ETIMEDOUT;
	}
	if (status & (1 << 1)) {
		printk("ERROR: erasing at address 0x%.8x: block locked error\n", offset);
		return -EPERM;
	}
	return 0;
}


/* Write a block of data to flash. Takes care of the word boundaries */
static int
omap_flash_write(loff_t to, size_t len, const u_char * buf)
{
	__u8 tmp[BUSWIDTH];
	int i, n, retlen = 0;
	//int retval = 0;

	/* First, we write a byte padded with original data until we reach a
	 * word boundary. */
	if (to & (BUSWIDTH - 1)) {
		__u32 aligned = to & ~(BUSWIDTH - 1);
		i = to - aligned;

		n = 0;
		down(&omap_flash_lock);
		*((__u16 *) tmp) = omap_flash_inw(aligned);
		up(&omap_flash_lock);

		while (len && i < BUSWIDTH)
			tmp[i++] = buf[n++], len--;

		if (omap_flash_write_word(aligned, *((__u16 *) tmp)))
		{
			return DCL_ERR_MFG_BLOCK_WRITE_ERROR;
			//return (-EIO);
		}

		to += n;
		buf += n;
		retlen += n;
	}

	/* Now we write words until we reach a non-word boundary. */
	while (len >= BUSWIDTH) {
		if (omap_flash_write_word(to, *((__u16 *) buf)))
		{
			return DCL_ERR_MFG_BLOCK_WRITE_ERROR;
			//return (-EIO);
		}

		to += BUSWIDTH;
		buf += BUSWIDTH;
		retlen += BUSWIDTH;
		len -= BUSWIDTH;
	}

	/* Top up the last unaligned bytes, padded with original data.... */
	if (len & (BUSWIDTH - 1)) {
		i = n = 0;

		down(&omap_flash_lock);
		*((__u16 *) tmp) = omap_flash_inw(to);
		up(&omap_flash_lock);

		while (len--)
			tmp[i++] = buf[n++];

		if (omap_flash_write_word(to, *((__u16 *) tmp)))
		{
			return DCL_ERR_MFG_BLOCK_WRITE_ERROR;
			//return (-EIO);
		}

		retlen += n;
	}
	
	return retlen;
//	return 0;
}


/* Write a flash word. The offset must be on a word boudary */
static int
omap_flash_write_word(__u32 offset, __u16 x)
{
	__u16 status;

	down(&omap_flash_lock);
	/* Check if Flash is (sufficiently) erased */
	if ((omap_flash_inw(offset) & x) != x) {
		printk("ERROR: cannot write, flash not erased at address 0x%.8x.\n", offset);
		up(&omap_flash_lock);
		return -EIO;	/*flash not erased */
	}

	omap_flash_outw(offset, 0x0050);	/* clear status register */
	omap_flash_outw(offset, 0x0040);	/* write setup */
	omap_flash_outw(offset, x);

	/* wait for program complete while polling the status register */
	while (((status = omap_flash_inw(offset)) & 0x0080) != 0x0080)
		schedule_timeout(1);

	omap_flash_outw(offset, 0x0050);	/* clear status register */
	omap_flash_outw(offset, 0x00FF);	/* restore read mode */
	up(&omap_flash_lock);

	/* process status register */
	if (status & (1 << 3)) {
		printk("ERROR: writing at address 0x%.8x: Vpp range error \n", offset);
		return -EPERM;
	}
	if (status & (1 << 4)) {
		printk("ERROR: writing at address 0x%.8x: program error \n", offset);
		return -ETIMEDOUT;
	}
	if (status & (1 << 1)) {
		printk("ERROR: writing at address 0x%.8x: device protect error\n", offset);
		return -EPERM;
	}

	return 0;
}


/*sdram test in kernel mode, return 0: successful, 1:fail*/
#define PAGES_TO_BE_TESTED 0x1800  /*test for 24MB, a page is 4KB*/
int sdram_test(void)
{
    static unsigned* upt[PAGES_TO_BE_TESTED];
    unsigned index,pagecnt,val;
    unsigned *addr, *start, *end;
    unsigned readback;
    int     rcode = 0;

    for (index=0;index<PAGES_TO_BE_TESTED;index++)
        upt[index] = 0;

    /*Disable interrupt before conducting sdram test at kernel mode. Can be markout if not care the execution time*/
    __asm__ (" mrs  r0, cpsr");
    __asm__ (" orr  r0, r0, #0xC0");
    __asm__ (" msr  cpsr_c, r0");

    /*Allocate all unused memory pages*/
    pagecnt = 0;
    for (index=0;index<PAGES_TO_BE_TESTED;index++) {
        upt[index] = (unsigned*)vmalloc(0x1000);
        if (upt[index]==0) {
            index++;
            break;
        }

        /*Now conduct memory cell test*/
        val = 0xaa5533cc;
        start = upt[index];
        end = start + 0x400;
        for (addr=start; addr<end; addr++) {
            *addr = val;
        }
        for (addr=start; addr<end; addr++) {
            readback = *addr;
            if ( readback != val) {
                printk ("\nTest error found at virtual address 0x%08X: read value: 0x%08X, expected value: 0x%08X\n", addr, readback, val);
                rcode = 1;
                return rcode;
            }
        }
            
        val = 0x55aacc33;
        start = upt[index];
        end = start + 0x400;
        for (addr=start; addr<end; addr++) {
            *addr = val;
        }
        for (addr=start; addr<end; addr++) {
            readback = *addr;
            if ( readback != val) {
                printk ("\nTest error found at virtual address 0x%08X: read value: 0x%08X, expected value: 0x%08X\n", addr, readback, val);
                rcode = 1;
                return rcode;
            }
        }

        /*Now conduct address line test (A7..0 is not necessary to test)*/
        start = upt[index];
        end = start + 0x400;
        for (addr=start; addr<end; addr = addr+64) {
            *addr = (uint)addr;
        }
        for (addr=start; addr<end; addr = addr+64) {
            readback = *addr;
            if ( readback != (uint)addr) {
                printk ("\nMemory address line test error found at virtual address 0x%08X: read value: 0x%08X, expected value: 0x%08X\n", addr, readback, (uint)addr);
                rcode = 1;
                return rcode;
            }
        }

    }
    pagecnt = index;

    /*Free all allocated memory pages*/
    for (index=0;index<pagecnt;index++) {
        if (upt[index]!=0) {
            vfree(upt[index]);
        }
    }
    
    /*Enable interrupt.  Can be markout if not care the execution time*/
    __asm__ (" mrs  r0, cpsr");
    __asm__ (" bic  r0, r0, #0xC0");
    __asm__ (" msr  cpsr_c, r0");

    printk("Test is successful.\n");
    return rcode;
}

/*
 * The ioctl() implementation
 */

int scull_ioctl(struct inode *inode, struct file *filp,
                 unsigned int cmd, unsigned long arg)
{

	int err = 0;
	int retval = 0;
	int val = 0;
	int i;
	unsigned int sum = 0;
	unsigned short *p;
	unsigned int test_result = 0;

	kmfg_block_T kmfg_block;

	
	/*
	 * extract the type and number bitfields, and don't decode
	 * wrong cmds: return ENOTTY (inappropriate ioctl) before access_ok()
	 */
	if (_IOC_TYPE(cmd) != SCULL_IOC_MAGIC) return -ENOTTY;
	if (_IOC_NR(cmd) > SCULL_IOC_MAXNR) return -ENOTTY;

	/*
	 * the direction is a bitmask, and VERIFY_WRITE catches R/W
	 * transfers. `Type' is user-oriented, while
	 * access_ok is kernel-oriented, so the concept of "read" and
	 * "write" is reversed
	 */
	if (_IOC_DIR(cmd) & _IOC_READ)
		err = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
	else if (_IOC_DIR(cmd) & _IOC_WRITE)
		err =  !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
	if (err) return -EFAULT;

	switch(cmd) {

	  case SCULL_IOCTSIZE: /* Tell: arg is the flash size */
		if (! capable (CAP_SYS_ADMIN))
			return -EPERM;
		flash_size = arg * 1024 * 1024;
		break;

	  case SCULL_IOCGCHKSUM:  //return checksum value of flash
		//printk("flash size=%x\n", flash_size);
		down(&omap_flash_lock);	
		for(i=0;i<flash_size;i=i+2)
		{
			if ((i<0x10000) || (i>=0x20000))
			{
				p= omap_nor_flash_base + i;
				/* add unsigned short value one by one */
				sum += (unsigned short)(*p);
			}
		}
		up(&omap_flash_lock);	
		retval = __put_user(sum, (unsigned int __user *)arg);
		break;

	  case SCULL_IOCSADDR: //debby added
	  	flash_start_addr = arg;
	  	break;

	  case SCULL_IOCSID: 
	  	retval = get_user(val, (int *)arg);
		//printk("SCULL_IOCSID val=%d\n", val);
		//printk("SCULL_IOCSID arg=%d\n", arg);
		//printk("retval=%d\n", retval);
	  	set_block_id = val;
		//printk("set_block_id=%d\n", set_block_id);
		
	  	//set_block_id = val;
	  	break;

	  case SCULL_IOCQSDRAM: //debby added
	  	test_result = sdram_test();
		retval = __put_user(test_result, (unsigned int __user *)arg);
	  	break;

	  case SCULL_IOCSESMCMD: //write UBoot ESM command
		{
#if 0  //should be further reviewed before normal use, stoneshih
			unsigned char *p = 0xc0000000;

			*p = 'd';*(p+1) = 'e';*(p+2) = 'a';*(p+3) = 'd';
			*(p+4) = 'b';*(p+5) = 'e';*(p+6) = 'e';*(p+7) = 'f';
			*(p+8) = arg;
#endif
		}
	  	break;

	  case SCULL_IOCGESMSTA: //get UBoot ESM cmd status
		{
/*
			unsigned char *p = &(E_UBOOT_DATA[0]);
			struct t_ESM *rval;	
			unsigned char q[4];
			unsigned long *aa;
			unsigned short *bb;

			rval = arg;	

			if ( (*p == 'b') && (*(p+1) == 'e') && (*(p+2) == 'e') && (*(p+3) == 'f') && (*(p+4) == 'd') && (*(p+5) == 'e') && (*(p+6) == 'a') && (*(p+7) == 'd') )
			{
				q[0]=*(p+8);q[1]=*(p+9);
				bb=&(q[0]);
				rval->esm_status = *bb;
				q[0]=*(p+10);q[1]=*(p+11);q[2]=*(p+12);q[3]=*(p+13);
				aa=&(q[0]);
				rval->fchecksum = *aa;

				*p=0x0;*(p+1)=0x0;*(p+2)=0x0;*(p+3)=0x0;
				*(p+4)=0x0;*(p+5)=0x0;*(p+6)=0x0;*(p+7)=0x0; 

				//Nick, note we cannot feed back the rval, otherwise
				//      the system will get address of rval
				//retval = __put_user(rval, (unsigned int __user *)arg);
			}
*/
		}
	  	break;

#ifdef KMFG_BLOCK
		case SCULL_IOC_GET_BLOCK:
		{

			#if 0 //for test
			printk("[SCULL_IOC_GET_BLOCK]:\n");

			
			if (copy_from_user(&kmfg_block, (unsigned int __user *)arg, sizeof(kmfg_block))) 
			{
				printk(KERN_ERR "SCULL_IOC_GET_BLOCK: CopyFrom User failed \n");
				return -EFAULT;
			}
			printk("[SCULL_IOC_GET_BLOCK]kmfg_block.block_id=%d, kmfg_block.block_size=%d\n", kmfg_block.block_id, kmfg_block.block_size);



			unsigned int __user * user_block_addr = kmfg_block.block;
			char kernel_block[7];

			if (copy_from_user(kernel_block, user_block_addr, 7)) 
			{
				printk(KERN_ERR "SCULL_IOC_GET_BLOCK: CopyFrom2 User failed \n");
				return -EFAULT;
			}
			printk("[SCULL_IOC_GET_BLOCK]kmfg_block.block=%s\n", kernel_block);
			

			kernel_block[0] = 'a';
			kernel_block[1] = 'c';
			kernel_block[2] = 'c';
			kernel_block[3] = 't';
			kernel_block[4] = 'o';
			kernel_block[5] = 'n';

			if (copy_to_user(user_block_addr, kernel_block, 7))
			{
				printk(KERN_ERR "SCULL_IOC_GET_BLOCK: CopyTo User failed \n");
				return -EFAULT;
			}
			#endif
			
			if (copy_from_user(&kmfg_block, (unsigned int __user *)arg, sizeof(kmfg_block))) 
			{
				printk(KERN_ERR "SCULL_IOC_GET_BLOCK: CopyFrom User failed \n");
				return -EFAULT;
			}
			printk("[SCULL_IOC_GET_BLOCK]kmfg_block.block_id=%d, kmfg_block.block_size=%d\n", kmfg_block.block_id, kmfg_block.block_size);

			unsigned int __user * user_block_addr = kmfg_block.block;

			size_t t_count = kmfg_block.block_size;
			set_block_id = kmfg_block.block_id;
			retval = kmfg_get_data(set_block_id, fdata, &t_count);
			if ((retval & DCL_ERR_MFG) == 0)
			{
				if(copy_to_user(user_block_addr, fdata, t_count))
				{
					printk(KERN_ERR "SCULL_IOC_GET_BLOCK: CopyTo User failed \n");
					return -EFAULT;
				}

			}
			else
			{
				printk(KERN_ERR "SCULL_IOC_GET_BLOCK: kmfg_get_data failed \n");
			}

			


			
			
		}
		break;
#endif

	  default:  /* redundant, as cmd was checked against MAXNR */
		return -ENOTTY;
	}
	return retval;

}


struct file_operations scull_fops = {
	.owner =    THIS_MODULE,
	.read =     scull_read,
	.write =    scull_write,
	.ioctl =    scull_ioctl,
	.open =     scull_open,
	.release =  scull_release,
};

/*
 * Finally, the module stuff
 */

/*
 * The cleanup function is used to handle initialization failures as well.
 * Thefore, it must be careful to work correctly even if some of the items
 * have not been initialized
 */
void scull_cleanup_module(void)
{

	/* Free flash data block */
	kfree(fdata);

#ifdef SCULL_DEBUG /* use proc only if debugging */
	scull_remove_proc();
#endif

	unregister_chrdev(scull_major, "flash_agent");
	return;
}


int  scull_init_module(void)
{


	int result;

//printk("flash_agent: omap_nor_flash_base=%x\n", omap_nor_flash_base);
	/*
	 * Get a range of minor numbers to work with, asking for a dynamic
	 * major unless directed otherwise at load time.
	*/
	//dev = MKDEV(scull_major, scull_minor);
	
	/* Using old style register to save programming effort */
	/* The limition is major number is not great than 256 */
	result = register_chrdev(scull_major, "flash_agent", &scull_fops);
	if (result < 0) {
	     printk("flash_agent: can't get major %d\n", scull_major);
	     return result;
	}

	/* allocate memory for flash data */
	//tot_len = system_len + contrast_len + brightness_len + battery_len + volume_len + wifi_len;
	tot_len = PRE_LENGTH + system_len + contrast_len + brightness_len + battery_len + volume_len + wifi_len;


	fdata = kmalloc(tot_len, GFP_KERNEL);
	if (!fdata) {
	   printk("flash_agent: memory allocation failed\n");
	   result = -ENOMEM;
	   goto fail;  /* Make this more graceful */
	}
 
#if 0
	/* get info data from flash device, and write to buffer */
	/* the kmfg put byte by byte to buffer */
{
	int i, len = tot_len;
	kmfg_get_data(KMFG_BLOCK_ID_ALL, fdata, &len);
	printk("fdata: ");
	for(i=0;i<20;i++) printk("%x ", *(fdata+i));
	printk("\n");
}
#endif

#ifdef SCULL_DEBUG /* only when debugging */
	scull_create_proc();
#endif

	return 0; /* succeed */

  fail:
	scull_cleanup_module();
	return result;


}


accton_module_init(scull_init_module);
module_exit(scull_cleanup_module);


MODULE_AUTHOR("Nick Wang");
MODULE_LICENSE("GPL");
