/*
 * scull.h -- definitions for the char module
 *
 */

#ifndef _SCULL_H_
#define _SCULL_H_

#include <linux/ioctl.h> /* needed for the _IOW etc stuff used later */

/*
 * Macros to help debugging
 */

#undef PDEBUG             /* undef it, just in case */
#ifdef SCULL_DEBUG
#  ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
#    define PDEBUG(fmt, args...) printk( KERN_DEBUG "scull: " fmt, ## args)
#  else
     /* This one for user space */
#    define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
#  endif
#else
#  define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif

#undef PDEBUGG
#define PDEBUGG(fmt, args...) /* nothing: it's a placeholder */

#ifndef SCULL_MAJOR
#define SCULL_MAJOR 253   /* dynamic major by default */
#endif

//Nick, put hardware information to 0x0c010000
//#define HW_POS	0x1fe0000
#define HW_POS	0x10000
#if 0
struct t_HIB{
    char macaddr[6];
    char productID[16];
    char manufacDate[8];
    char HwRevision[4];
    char product_name[16];
    char hw_id[1];
}  __attribute__ ((packed)) *fdata;
#endif
unsigned char *fdata;

//Define the ESM status structure
struct t_ESM{
    unsigned int esm_status;
    unsigned long fchecksum;
} __attribute__ ((packed));


/*
 * Split minors in two parts
 */
#define TYPE(minor)	(((minor) >> 4) & 0xf)	/* high nibble */
#define NUM(minor)	((minor) & 0xf)		/* low  nibble */


/*
 * The different configurable parameters
 */
extern int scull_major;     /* main.c */
extern int scull_nr_devs;
extern int scull_quantum;
extern int scull_qset;

extern int scull_p_buffer;	/* pipe.c */


/*
 * Prototypes for shared functions
 */
#if 0
int     scull_p_init(dev_t dev);
void    scull_p_cleanup(void);
int     scull_access_init(dev_t dev);
void    scull_access_cleanup(void);

int     scull_trim(struct scull_dev *dev);
#endif
ssize_t scull_read(struct file *filp, char __user *buf, size_t count,
                   loff_t *f_pos);
ssize_t scull_write(struct file *filp, const char __user *buf, size_t count,
                    loff_t *f_pos);
loff_t  scull_llseek(struct file *filp, loff_t off, int whence);
int     scull_ioctl(struct inode *inode, struct file *filp,
                    unsigned int cmd, unsigned long arg);


/*
 * Ioctl definitions
 */

/* Use 0xFA as magic number */
#define SCULL_IOC_MAGIC  0xFA

#define SCULL_IOCRESET    _IO(SCULL_IOC_MAGIC, 0)

/*
 * S means "Set" through a ptr,
 * T means "Tell" directly with the argument value
 * G means "Get": reply by setting through a pointer
 * Q means "Query": response is on the return value
 * X means "eXchange": switch G and S atomically
 * H means "sHift": switch T and Q atomically
 */
#define SCULL_IOCTSIZE	_IOW(SCULL_IOC_MAGIC,  1, int)
#define SCULL_IOCGCHKSUM	_IOR(SCULL_IOC_MAGIC,  2, unsigned int)
#define SCULL_IOCSADDR	_IOW(SCULL_IOC_MAGIC,  3, unsigned long) //debby added
#define SCULL_IOCQSDRAM	_IOR(SCULL_IOC_MAGIC,  4, unsigned int) //debby added
#define SCULL_IOCSESMCMD	_IOW(SCULL_IOC_MAGIC,  5, unsigned short) 
#define SCULL_IOCGESMSTA	_IOR(SCULL_IOC_MAGIC,  6, unsigned int) 
#define SCULL_IOCSID	_IOW(SCULL_IOC_MAGIC,  7, unsigned int) 

#define SCULL_IOC_MAXNR 7

#endif /* _SCULL_H_ */
