/*
 * linux/drivers/video/NT7523vfb.c -- NOVATEK CMOS LCD Controller frame buffer device
 *
 * Modified from pcf8833vfb.c (from Alan@Arcadyan) at Oct. 5 2005 by Michael Ruan (michael_ruan@accton.com.tw)
 *
 * This version just supports vfb_mmap, but not vfb_write and vfb_copy
 *
 */
 
/*
 *  linux/drivers/video/vfb.c -- Virtual frame buffer device
 *
 *      Copyright (C) 2002 James Simmons
 *
 *	Copyright (C) 1997 Geert Uytterhoeven
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <linux/fb.h>
#include <linux/init.h>

/*vvv Alan Add vvv*/
/* Support for PCF8833 at OMAP 1710 Platform */
#include <asm/io.h>
#include <asm/arch/hardware.h>
#include <asm/arch/gpio.h>
#include <asm/arch/mux.h>

/* Michael add support for DMA*/
#include <asm/dma.h>

// #define OMAPFB_DBG_FIFO 1
#define OMAPFB_DBG 9

#include "debug.h"
/*^^^ Alan Add ^^^*/

    /*
     *  RAM we reserve for the frame buffer. This defines the maximum screen
     *  size
     *
     *  The default can be overridden if the driver is compiled as a module
     */
#define WIFI_PHONE_LCD

//#define DEBUG_PM
#ifdef DEBUG_PM
#define info(format, arg...) 	PRINT_MSG(KMSG_MSG_LEVEL_INFOMATION, "nt7523", format...)
#define dbg(format, arg...)	PRINT_MSG(KMSG_MSG_LEVEL_DEBUG, "nt7523", format...)
#else
#define info(format, arg...)
#define dbg(format, arg...)
#endif	//DEBUG_PM

//Michael
#define LCD_WIDTH               128
#define LCD_HEIGHT              160
#define LCD_PIXEL_COUNT	LCD_WIDTH * LCD_HEIGHT 

#define LCD_BITS_PER_PIXEL      16
#define LCD_FRAME_BUFFER_BYTES  ( LCD_WIDTH * LCD_HEIGHT * LCD_BITS_PER_PIXEL / 8 )
//#define VIDEOMEMSIZE	(1*2*128*128)	/* 8 KB  In Byte */
#define VIDEOMEMSIZE LCD_FRAME_BUFFER_BYTES

//#define OSCILLATION_USEC 200000
//#define COMMON_USEC      5000
#define OSCILLATION_MSEC 200
#define COMMON_MSEC      5


#undef DEBUG
//#define DEBUG
#ifdef DEBUG
#define DPRINTK(ARGS...)  printk(KERN_INFO "<%s>: ",__FUNCTION__);printk(ARGS)
#define FN_IN printk(KERN_INFO "[%s]: start\n", __FUNCTION__)
#define FN_OUT(n) printk(KERN_INFO "[%s]: end(%u)\n",__FUNCTION__, n)
#else

#define DPRINTK( x... )
#define FN_IN
#define FN_OUT(x)
#endif


//Add by Owen Lin
//Michael:15fps
//#define PCF8833_TIMER_WUT   ( jiffies + ( HZ / 15 ) )

//Michael:10fps
#define PCF8833_TIMER_WUT   ( jiffies + ( HZ/10) )

//Use Kernel Thread to replace Timer
static  long    gUpdateThreadPid = 0;
struct  semaphore gUpdateSem;
struct  semaphore gUpdateSem2;

static int lcd_update_thread(void *data);


static void *videomemory;
static u_long videomemorysize = VIDEOMEMSIZE;


//Add by Michael: 0 ->false
static char isWriting = 0;

#ifdef CONFIG_PM
struct pm_dev *lcm_pm_dev = NULL;
#endif 


#if 0
/*
static char COLORCOUNT = 5;
static int COLORARRAY[5] = {0xF800, 0x07E0, 0x001F, 0xFFFF, 0x0000};

static int RED = 0xF800;
static int GREEN = 0x07E0;
static int BLUE = 0x001F;
static int WHITE = 0xFFFF;
static int BLACK = 0x0000;
*/
#endif

#define LCD_DMA
//#define LCD_DMA_TEST


#ifdef LCD_DMA

int dma_lch_tx;
//char *dmabuf_tx;
__u16 *dmabuf_tx;

dma_addr_t dmaphys_tx;

static __inline__ void lcd_dma_callback(int lch,u16 ch_status, void* data);

#endif

#ifdef LCD_DMA_TEST

__u16 *dmabuf_tx2;
dma_addr_t dmaphys_tx2;

#endif


MODULE_PARM(videomemorysize, "l");

static struct fb_var_screeninfo vfb_default __initdata = {


        .xres =             LCD_WIDTH,
        .yres =             LCD_HEIGHT,
        .xres_virtual =     LCD_WIDTH,
        .yres_virtual =     LCD_HEIGHT,
        .bits_per_pixel =   LCD_BITS_PER_PIXEL,

        .red =              { 0, 5, 0 }, /* Use RGB565 Color setting */
        .green =            { 5, 6, 0 },
        .blue =             { 11, 5, 0 },
        .transp =           { 0, 0, 0 },
        .activate =         FB_ACTIVATE_TEST,
        .height =           -1,
        .width =            -1,
        .pixclock =         20000,
        .left_margin =      64,
        .right_margin =     64,
        .upper_margin =     32,
        .lower_margin =     32,
        .hsync_len =        64,
        .vsync_len =        2,
        .vmode =            FB_VMODE_NONINTERLACED,
};

static struct fb_fix_screeninfo vfb_fix __initdata = {
        .id =           "NT7523 FB",
        .type =         FB_TYPE_PACKED_PIXELS,
        //.visual =       FB_VISUAL_PSEUDOCOLOR,
        //.visual =       FB_VISUAL_TRUECOLOR,        
        .visual =       FB_VISUAL_DIRECTCOLOR, 
        .xpanstep =     1,
        .ypanstep =     1,
        .ywrapstep =    1,
        .line_length =  128*2,
        .accel =        FB_ACCEL_NONE,
        .smem_len = VIDEOMEMSIZE,	/* In Byte */
};

static int vfb_enable __initdata = 0;	/* disabled by default */
MODULE_PARM(vfb_enable, "i");

/* Michael Define for NT7352 at OMAP */
static volatile __u16 *omap_8833fb_base = NULL;
static volatile __u16 *omap_8833fb_base_data = NULL;

static struct fb_info *info_saved = NULL;
static struct timer_list pcf8833_timer;

#define OMAP_PCF8833_CSTN_START         0x0A000000 /* CS2B */
//#define OMAP_PCF8833_CSTN_START         0x04000000 /* CS1A for vPhone*/
#define OMAP_PCF8833_CSTN__SIZE        LCD_FRAME_BUFFER_BYTES

//Write indexs into IR
#define OMAP_PCF8833_CSTN_COMMAND       (*omap_8833fb_base) /* set FADD_1 as 0 */

//Write control registers or RAM data
#define OMAP_PCF8833_CSTN_DATA          (*omap_8833fb_base_data) /* set FADD_1 as 1 */



 #define COMMAND_MEM_MAP(REG) \
 				OMAP_PCF8833_CSTN_COMMAND = ((REG>> 8 ) & 0xff); \
				OMAP_PCF8833_CSTN_COMMAND = REG; 


 #define DATA_MEM_MAP(DATA) \
 				OMAP_PCF8833_CSTN_DATA = ((DATA>> 8 ) & 0xff); \
				OMAP_PCF8833_CSTN_DATA = DATA;


#ifdef WIFI_PHONE_LCD
static struct timer_list lcm_timer;
static int lcm_timer_active = 0;
static int default_lcmbklight_timeval = 60;		//default 1 min
static int lcm_backlight_status = 1;
void add_lcm_backlight_timer(int interval);

#define GPTIMER_BASE 0xfffb1400
	#define TCLR		(GPTIMER_BASE + 0x24)	//timer control register
		#define 	TCLR_ST		(1 << 0)		//start timer
		#define 	TCLR_AR	(1 << 1)		//auto-reload / one-shot
		#define 	TCLR_CE		(1 << 6)		//compare mode enable
		#define 	TCLR_SCPWM	(1 << 7)	//negative / postive pulse
		#define 	TCLR_TRG_ON_OF 	( 1 << 10)	//trigger on overflow
		#define 	TCLR_TRG_ON_OF_MT	(2 << 10)	//trigger on overflow & match
		#define	TCLR_PT		(1 << 12)	//pulse / toggle
	#define TCRR		(GPTIMER_BASE + 0x28)
	#define TLDR		(GPTIMER_BASE + 0x2c)
	#define TMAR		(GPTIMER_BASE + 0x38)
#if defined(VM1185T_R01) || defined(VD1185T_R01)
void set_lcd_brightness(int n)	//0 - 255
{
	unsigned int value;
	omap_cfg_reg(M18_PWM0);
	omap_writel(0xffffff00, TLDR);
	omap_writel(0xffffffff - n , TMAR);	
	omap_writel(0xffffffff, TCRR);
	value = omap_readl(TCLR) | TCLR_PT | TCLR_TRG_ON_OF_MT | TCLR_CE | TCLR_AR | TCLR_ST & (~TCLR_SCPWM);
	omap_writel(value , TCLR);
}
EXPORT_SYMBOL(set_lcd_brightness);
#endif

#ifdef VM1185T_R01
#define LCM_BACKLIGHT_GPIO 	2
#endif

#ifdef VM1185T_R0B
#define LCM_BACKLIGHT_GPIO	34
#endif

#ifdef VD1185T_R01
#define LCM_BACKLIGHT_GPIO	59
#endif

void set_lcd_backlight(int enable)
{
#if defined(VM1185T_R01) || defined(VD1185T_R01)
	omap_set_gpio_dataout(LCM_BACKLIGHT_GPIO, enable ? 1 : 0);
#endif
#ifdef VM1185T_R0B
	omap_set_gpio_dataout(LCM_BACKLIGHT_GPIO, enable ? 0 : 1);
#endif
	lcm_backlight_status = enable ? 1 : 0;
}
EXPORT_SYMBOL(set_lcd_backlight);
int get_lcm_backlight_status()
{
	return lcm_backlight_status;
}
EXPORT_SYMBOL(get_lcm_backlight_status);
static void lcm_backlight_timer(unsigned long data)
{
	set_lcd_backlight(0);		//turn off LCM backlight
	del_timer(&lcm_timer);		//remove the timer
	lcm_timer_active = 0;
}
static void add_lcm_backlight_timer(int interval)
{
	if(interval == 0)
		return;
	if(lcm_timer_active)
		mod_timer(&lcm_timer, jiffies + interval * HZ);
	else
	{	
		init_timer(&lcm_timer);
		lcm_timer.function = lcm_backlight_timer;
		lcm_timer.expires = jiffies + interval * HZ;
		lcm_timer.data = 0;
		add_timer(&lcm_timer);
		lcm_timer_active = 1;
	}
}
void del_lcmbklight_timer()
{
	if(lcm_timer_active)
		del_timer(&lcm_timer);
}
EXPORT_SYMBOL(del_lcmbklight_timer);

void add_lcmbklight_timer()
{
	if(default_lcmbklight_timeval == 0) {	//if =0 do not turn off LCM backlight
		del_lcmbklight_timer();
		return;
	}
	add_lcm_backlight_timer(default_lcmbklight_timeval);
}
EXPORT_SYMBOL(add_lcmbklight_timer);

void set_lcmbklight_timeval(int timeval)
{
	default_lcmbklight_timeval = timeval;
	add_lcmbklight_timer();
}
EXPORT_SYMBOL(set_lcmbklight_timeval);
int get_lcmbklight_timeval()
{
	return default_lcmbklight_timeval;
}
EXPORT_SYMBOL(get_lcmbklight_timeval);
#endif	//WIFI_PHONE_LCD

/* Michael Library functions for NT7523 at OMAP */

static void omap_8833fb_setX( __u16 dxs, __u16 dxe )
{

	OMAP_PCF8833_CSTN_COMMAND = ((0x0016 >> 8)& 0xff) ;
	OMAP_PCF8833_CSTN_COMMAND = 0x0016;

	OMAP_PCF8833_CSTN_DATA = dxe;
	OMAP_PCF8833_CSTN_DATA = dxs;

}

static void omap_8833fb_setY( __u16 dys, __u16 dye )
{

	OMAP_PCF8833_CSTN_COMMAND = ((0x0017>> 8)& 0xff);
	OMAP_PCF8833_CSTN_COMMAND = 0x0017;

	OMAP_PCF8833_CSTN_DATA = dye;
	OMAP_PCF8833_CSTN_DATA = dys;

}

#if 0
void omap_8833fb_copy(struct fb_info *p, const struct fb_copyarea *area) 
{
/*
 *      @dx: The x and y coordinates of the upper left hand corner of the
 *	@dy: destination area on the screen.
 *      @width: How wide the rectangle is we want to copy.
 *      @height: How tall the rectangle is we want to copy.
 *      @sx: The x and y coordinates of the upper left hand corner of the
 *      @sy: source area on the screen.
 */
	FN_IN;
	__u32 dxs, dxe;
	__u32 dys, dye;
	__u32 sx, height;
	__u16 *src, *srcX;

	DBGENTER(1);

	 /* Set the start/end point of X, Y axis */
	dxs = ( area->dx & 0x00ff );
	dxe = ( ( dxs + area->width ) & 0x00ff );
	dys = ( area->dy & 0x00ff );
	dye = ( (dys + area->height) & 0x00ff );
	omap_8833fb_setX( dxs, dxe );
	omap_8833fb_setY( dys, dye );

	sx = area->sx * 2;
	srcX = src = (__u16 *) ( p->screen_base + sx + area->sy * p->fix.line_length );
//        DPRINTK( "screen_base = 0x%08x\n", (__u32) p->screen_base );
//        DPRINTK( "srcX = 0x%08x\n", (__u32)srcX );

	height = area->height;

	//OMAP_PCF8833_CSTN_FILL_FB_START;

	while (height--) {
		int x;
                        
		 /* Assume the color format is RGB565 */
		for ( x = area->width; x > 0; --x ) {
			OMAP_PCF8833_CSTN_DATA = (((*srcX)>>8)&0xff);
			OMAP_PCF8833_CSTN_DATA = ((*srcX)&0xff);
			srcX++;                        
		}
		src += p->fix.line_length;
		srcX = src;
	}

	//OMAP_PCF8833_CSTN_FILL_FB_END;
	DBGLEAVE(1);
}

#endif

static void omap_8833fb_timer( unsigned long data ) {


//	FN_IN;

	if(likely(isWriting))
	{
	up( &gUpdateSem );
	}
	pcf8833_timer.expires = PCF8833_TIMER_WUT;
      add_timer( & pcf8833_timer );


}

static void omap_8833fb_chip_init(void)
{
	//DBGENTER(1);
	FN_IN;
	/* Start the initialization process of NT7532 */
	//Write upper 8 bits first because we use 8bit mode
	OMAP_PCF8833_CSTN_COMMAND = ((0x0000 >> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_COMMAND = 0x0000;
	OMAP_PCF8833_CSTN_DATA = ((0x0001 >> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0001;
	//udelay(OSCILLATION_USEC);
	mdelay(OSCILLATION_MSEC);


	OMAP_PCF8833_CSTN_COMMAND = ((0x0002 >> 8 ) & 0xff); 	
	OMAP_PCF8833_CSTN_COMMAND = 0x0002; 
	OMAP_PCF8833_CSTN_DATA = ((0x0400 >> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0400;
	//udelay(COMMON_USEC);
	mdelay(COMMON_MSEC);
	
	OMAP_PCF8833_CSTN_COMMAND = ((0x0001>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0001; 
	OMAP_PCF8833_CSTN_DATA = ((0x0213 >> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0213;
	mdelay(COMMON_MSEC);

	OMAP_PCF8833_CSTN_COMMAND = ((0x0002>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_COMMAND = 0x0002;
	OMAP_PCF8833_CSTN_DATA = ((0x0308>> 8 ) & 0xff);	
	OMAP_PCF8833_CSTN_DATA = 0x0308;
	mdelay(COMMON_MSEC);

	OMAP_PCF8833_CSTN_COMMAND = ((0x000C>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_COMMAND = 0x000C;	
	OMAP_PCF8833_CSTN_DATA = ((0x8000>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x8000;
	mdelay(COMMON_MSEC);

//	OMAP_PCF8833_CSTN_COMMAND = ((0x0003>> 8 ) & 0xff); 
//	OMAP_PCF8833_CSTN_COMMAND = 0x0003; 
//	OMAP_PCF8833_CSTN_DATA = ((0x6C04>> 8 ) & 0xff);
//	OMAP_PCF8833_CSTN_DATA = 0x6C04;
//	udelay(COMMON_USEC);

//new--

      OMAP_PCF8833_CSTN_COMMAND = ((0x0003>> 8 ) & 0xff);
      OMAP_PCF8833_CSTN_COMMAND = 0x0003;
      OMAP_PCF8833_CSTN_DATA = ((0x6804>> 8 ) & 0xff);
      OMAP_PCF8833_CSTN_DATA = 0x6804;
      //udelay(50000);
	mdelay(50);
	  
      OMAP_PCF8833_CSTN_COMMAND = ((0x0003>> 8 ) & 0xff);
      OMAP_PCF8833_CSTN_COMMAND = 0x0003;
      OMAP_PCF8833_CSTN_DATA = ((0x6a04>> 8 ) & 0xff);
      OMAP_PCF8833_CSTN_DATA = 0x6a04;
      //udelay(50000);
	mdelay(50);

      OMAP_PCF8833_CSTN_COMMAND = ((0x0003>> 8 ) & 0xff);
      OMAP_PCF8833_CSTN_COMMAND = 0x0003;
      OMAP_PCF8833_CSTN_DATA = ((0x6C04>> 8 ) & 0xff);
      OMAP_PCF8833_CSTN_DATA = 0x6C04;
      //udelay(100000);
	mdelay(100);


//end--



	OMAP_PCF8833_CSTN_COMMAND = ((0x0004>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0004; 
//	OMAP_PCF8833_CSTN_DATA = ((0x033f>> 8 ) & 0xff);
//	OMAP_PCF8833_CSTN_DATA = 0x033f;
	OMAP_PCF8833_CSTN_DATA = ((0x0343>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0343;
	mdelay(COMMON_MSEC);

	OMAP_PCF8833_CSTN_COMMAND = ((0x0005>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0005; 
	OMAP_PCF8833_CSTN_DATA = ((0x0030>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0030;
	mdelay(COMMON_MSEC);

	OMAP_PCF8833_CSTN_COMMAND = ((0x0006>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0006; 
	OMAP_PCF8833_CSTN_DATA = ((0x0000>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0000;
	mdelay(COMMON_MSEC);


	OMAP_PCF8833_CSTN_COMMAND = ((0x0007>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0007; 
//	OMAP_PCF8833_CSTN_DATA = ((0x0103>> 8 ) & 0xff);
//	OMAP_PCF8833_CSTN_DATA = 0x0103;
      OMAP_PCF8833_CSTN_DATA = ((0x0003>> 8 ) & 0xff);
      OMAP_PCF8833_CSTN_DATA = 0x0003;
	mdelay(COMMON_MSEC);



	OMAP_PCF8833_CSTN_COMMAND = ((0x000B>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x000B; 
	OMAP_PCF8833_CSTN_DATA = ((0x0000>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0000;
	mdelay(COMMON_MSEC);

	OMAP_PCF8833_CSTN_COMMAND = ((0x0016>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0016; 
	OMAP_PCF8833_CSTN_DATA = ((0x7f00>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x7f00;
	mdelay(COMMON_MSEC);

	OMAP_PCF8833_CSTN_COMMAND = ((0x0017>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0017; 
	OMAP_PCF8833_CSTN_DATA = ((0x9F00>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x9F00;
	mdelay(COMMON_MSEC);
	
	OMAP_PCF8833_CSTN_COMMAND = ((0x0021>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0021; 
	OMAP_PCF8833_CSTN_DATA = ((0x0000>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0000;
	mdelay(COMMON_MSEC);

	OMAP_PCF8833_CSTN_COMMAND = ((0x0007>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0007; 
	OMAP_PCF8833_CSTN_DATA = ((0x0003>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0003;
	mdelay(COMMON_MSEC);


	//Michael:Avoid random screen
/*
	COMMAND_MEM_MAP(0x0022);
	int index;
	//128*160

	
	for( index = 0; index < 20480; ++ index ) 
	{
		DATA_MEM_MAP(0x0000);
	}
        Grayscale palette
	
        COMMAND_MEM_MAP(0x0030);
        DATA_MEM_MAP(0x0300);
        COMMAND_MEM_MAP (0x0031);
        DATA_MEM_MAP(0x0806);
        COMMAND_MEM_MAP(0x0032);
        DATA_MEM_MAP(0x0c0a);
        COMMAND_MEM_MAP(0x0033);
        DATA_MEM_MAP(0x0f0e);
        COMMAND_MEM_MAP (0x0034);
        DATA_MEM_MAP (0x1110);
        COMMAND_MEM_MAP (0x0035);
        DATA_MEM_MAP (0x1312);
        COMMAND_MEM_MAP (0x0036);
        DATA_MEM_MAP (0x1514);
        COMMAND_MEM_MAP (0x0037);
        DATA_MEM_MAP (0x1716);
        COMMAND_MEM_MAP (0x0038);
        DATA_MEM_MAP (0x1918);
        COMMAND_MEM_MAP (0x0039);
        DATA_MEM_MAP (0x1b1a);
        COMMAND_MEM_MAP (0x003A);
        DATA_MEM_MAP (0x1d1c);
        COMMAND_MEM_MAP (0x003B);
        DATA_MEM_MAP (0x201e);
        COMMAND_MEM_MAP (0x003C);
        DATA_MEM_MAP (0x2422);
        COMMAND_MEM_MAP (0x003D);
        DATA_MEM_MAP (0x2826);
        COMMAND_MEM_MAP (0x003E);
        DATA_MEM_MAP (0x2e2b);
        COMMAND_MEM_MAP (0x003F);
        DATA_MEM_MAP (0x3431);
*/
	//DBGLEAVE(1);
}


/* Check the existing of PCF8833 through OMAP1710 memory bus */
static int omap_8833fb_probe(void)
{

	FN_IN;


	//DBGENTER(1);
	//LCD reset pin
        omap_cfg_reg( L14_1710_GPIO49 );
        omap_request_gpio( 49 );
        omap_set_gpio_direction( 49, 0 );
	 //omap_set_gpio_dataout( 49, 1 );        
        //udelay( 200 );
        omap_set_gpio_dataout( 49, 0 );
        udelay( 100 );
        omap_set_gpio_dataout( 49, 1 );        
        DPRINTK( "Michael:omap_7523fb_probe reset complete!\n" );

	//LCD dim pin -- Still have problems
        //omap_cfg_reg( W13_1710_GPIO55 );

#ifdef VM1185T_R01
	//R01
	//Turn-on LCM Backlight
        omap_cfg_reg( D15_1710_GPIO2);
        omap_request_gpio( 2 );
  	omap_set_gpio_direction( 2, 0 );
        omap_set_gpio_dataout( 2, 1 );
#endif
#ifdef VM1185T_R0B
	omap_cfg_reg( A17_1710_GPIO34 );
	omap_request_gpio( 34 );
  	omap_set_gpio_direction( 34, 0 );
	//duty cycle 0
	omap_set_gpio_dataout( 34, 0 );
#endif
#ifdef VD1185T_R01
	omap_cfg_reg( L3_1710_GPIO59);
    omap_request_gpio( 59 );
  	omap_set_gpio_direction( 59, 0 );
	omap_set_gpio_dataout( 59, 1 );	
#endif


/*        
        udelay( 100 );
        omap_set_gpio_dataout( 34, 1 ); 
*/
	//Michael:Need more config.
	//LCD_W E/WRB
       omap_cfg_reg( M8_1710_CS2UWE ); //is 	W8 suitable?
/*
       omap_cfg_reg( M8_1710_GPIO60); //is 	W8 suitable?
        omap_request_gpio( 60);
  	omap_set_gpio_direction( 60, 0 );
        omap_set_gpio_dataout( 60, 0 );
*/

	//LCD_R L/R(RDB)


#if defined(VM1185T_R01) || defined(VD1185T_R01)
	//R01
	//LCD_R L/R(RDB)
        omap_cfg_reg( L3_1710_GPIO59);
        omap_request_gpio( 59 );
  	omap_set_gpio_direction( 59, 0 );
	//Enable to set WRITE default
        omap_set_gpio_dataout( 59, 1 );	

	//R01
	//Direction of The Level Shift
        omap_cfg_reg( W6_1710_GPIO12);
        omap_request_gpio( 12 );
  	omap_set_gpio_direction( 12, 0 );
	//Enable to set WRITE default
        omap_set_gpio_dataout( 12, 1 );	

	//R01
	//Output Enable of Level Shift
        omap_cfg_reg( Y1_1710_GPIO16);
        omap_request_gpio( 16 );
  	omap_set_gpio_direction( 16, 0 );
	//Enable to set WRITE default
        omap_set_gpio_dataout( 16, 0 );	
#endif
#if defined(VM1185T_R01) || defined(VD1185T_R01)
	omap_cfg_reg(M18_PWM0);
	set_lcd_brightness(128);		//50% brightness
#endif
       DPRINTK( "Michael:omap_7523fb_probe Write GPIO complete!\n" );

	//DBGLEAVE(1);

        return 1;
}


/* Initial the OMAP1710 memory bus */
static int omap_8833fb_bus_init(void)
{

	//DBGENTER(1);
	
	FN_IN;

	/* Get & lock the virtual address */
	omap_8833fb_base = (__u16 *) ioremap(OMAP_PCF8833_CSTN_START, OMAP_PCF8833_CSTN__SIZE); 
	//omap_8833fb_base_data = (__u16 *) ioremap(OMAP_PCF8833_CSTN_START + 0x01, OMAP_PCF8833_CSTN__SIZE); 
	omap_8833fb_base_data = omap_8833fb_base + 0x01;

	DPRINTK( "omap_8833fb_base= 0x%08x; omap_8833fb_base_data = 0x%08x\n", omap_8833fb_base, omap_8833fb_base_data );

#ifdef LCD_DMA

/**/
	//Set up DMA Logical Channel for Tx 
	if(omap_request_dma(0,"EMIFS_CS2B Tx", lcd_dma_callback, NULL, &dma_lch_tx) < 0) {
		DPRINTK("Michael:Error in DMA:EMIFS_CS2B Tx\n"); 
		return -1;
	}
	

#ifdef LCD_DMA_TEST

/*
	dmabuf_tx = dma_alloc_coherent(NULL, 4, &dmaphys_tx, GFP_KERNEL | GFP_DMA);
	dmabuf_tx2 = dma_alloc_coherent(NULL, 4, &dmaphys_tx2, GFP_KERNEL | GFP_DMA);
*/

	dmabuf_tx = dma_alloc_coherent(NULL, OMAP_PCF8833_CSTN__SIZE, &dmaphys_tx, GFP_KERNEL | GFP_DMA);
	dmabuf_tx2 = dma_alloc_coherent(NULL, OMAP_PCF8833_CSTN__SIZE, &dmaphys_tx2, GFP_KERNEL | GFP_DMA);

	if((!dmabuf_tx) || (!dmabuf_tx2)){
		DPRINTK("Michael:Cannot allocate DMA for EMIFS_CS2B Tx");
		return -1;
	}

	omap_set_dma_dest_params(dma_lch_tx, OMAP_DMA_PORT_EMIFF,	 OMAP_DMA_AMODE_POST_INC, dmaphys_tx2);



#else

	//dmabuf_tx = dma_alloc_coherent(NULL, OMAP_PCF8833_CSTN__SIZE, &dmaphys_tx, GFP_KERNEL | GFP_DMA);
	dmabuf_tx = dma_alloc_coherent(NULL, OMAP_PCF8833_CSTN__SIZE, &dmaphys_tx, GFP_KERNEL | GFP_DMA);

	if(!dmabuf_tx) {
		DPRINTK("Michael:Cannot allocate DMA for EMIFS_CS2B Tx");
		return -1;
	}

	omap_set_dma_dest_params(dma_lch_tx, OMAP_DMA_PORT_EMIFS,	 OMAP_DMA_AMODE_CONSTANT, (unsigned long)(OMAP_PCF8833_CSTN_START+0x02));
	//omap_set_dma_dest_params(dma_lch_tx, OMAP_DMA_PORT_EMIFS,	 OMAP_DMA_AMODE_CONSTANT, (unsigned long)io_v2p(omap_8833fb_base));
	//omap_set_dma_dest_params(dma_lch_tx, OMAP_DMA_PORT_EMIFS,	 OMAP_DMA_AMODE_CONSTANT, (unsigned long)(io_v2p(omap_8833fb_base_data)));

#endif

#endif






	//Michael:Taichung's bootloader have done init
       /*	
	if (omap_8833fb_probe() < 0) {
		iounmap((void *) omap_8833fb_base);
		DPRINTK( "Michael: Enter omap_7523fb_bus_init_fail\n" );
		return (-ENXIO);
	}

	//Do the initial proces of NT7523
	omap_8833fb_chip_init();
	*/
	//Michael:Taichung's bootloader have done init




	DPRINTK( "Michael: Enter omap_7523fb_bus_init_ok\n" );
	//DBGLEAVE(1);
	return 1;
}


#if 0
//Alan: Assume the count is pixel count in byte
static ssize_t omap_8833fb_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{

	FN_IN;
	unsigned long p = *ppos;
	unsigned long total_size;
	struct fb_copyarea area;
	u32 *dst;
	int i, cnt = 0;

#ifdef TEST_COMMAND
        int height = 128, width = 128;
        int x, y; 

	omap_8833fb_setX( 0, 128 );
	omap_8833fb_setY( 0, 128 );
	OMAP_PCF8833_CSTN_FILL_FB_START;
	for( y = 0; y < height; ++y ) {
//                __u16 data = ( 0x1000 >> ( height / 8 ) );
//                __u16 data = 0xF800;
                __u16 data = 0x07E0;
		 /* Assume the color format is RGB565 */
		for ( x = 0; x < width; ++x ) {
			OMAP_PCF8833_CSTN_DATA = (( data >> 8 ) & 0xff);
			OMAP_PCF8833_CSTN_DATA = (( data) & 0xff);
		}
	}
	OMAP_PCF8833_CSTN_FILL_FB_END;
        return count;

        // end of test
#endif

	DBGENTER(1);

	if ( (info_saved == NULL) ||  (count == 0) ) return -ENOSPC;

	total_size = info_saved->screen_size;
	if (total_size == 0)
		total_size = info_saved->fix.smem_len;

	if (p > total_size)
	    return -ENOSPC;
	if (count >= total_size)
	    count = total_size;
	if (count + p > total_size) {
	    count = total_size - p;
	}

//        DPRINTK( "p = %d, count = %d total_size = %d\n", (u32)p, (u32)count, (u32)total_size );

	//p is byte offset
	dst = (u32 *) (info_saved->screen_base + p);
	copy_from_user( dst, buf, count);

	/* Setup the copyarea region */
	area.dy = p/info_saved->fix.line_length;	// Y axis start from 0
	area.dx = (p - (p/info_saved->fix.line_length)*info_saved->fix.line_length) / 2;
	area.width = info_saved->var.xres - area.dx;
	if ( area.width > count/2 ) area.width = count/2;
	area.height = 1;
	area.sx = area.dx;
	area.sy = area.dy;

//        DPRINTK( "dx = %d, dy = %d width = %d\n", area.dx, area.dy, area.width );

	omap_8833fb_copy( info_saved, &area );
	cnt = area.width * 2; //16 bits


	//Write Line by line into FrameBuffer
	while (  (count - cnt) > info_saved->fix.line_length ) {
		area.dy++;
		area.width = info_saved->var.xres;
		area.height = 1;
		area.dx = area.sx = 0;
		area.sy = area.dy;
		omap_8833fb_copy( info_saved, &area );
		cnt += info_saved->fix.line_length;
		//DPRINTK( "count = %d, cnt = %d\n", count, cnt );
	}
	if ( (count -= cnt) != 0 ) {
		area.dy++;
		area.width = count/2;
		area.height = 1;
		area.dx = area.sx = 0;
		area.sy = area.dy;
		omap_8833fb_copy( info_saved, &area );
		cnt += count;
	}

	DBGLEAVE(1);

	return cnt;
}

#endif
    /*
     *  Interface used by the world
     */
static int vfb_init(void);
static int vfb_setup(char *);

static int vfb_check_var(struct fb_var_screeninfo *var,
			 struct fb_info *info);
static int vfb_set_par(struct fb_info *info);
static int vfb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
			 u_int transp, struct fb_info *info);
static int vfb_pan_display(struct fb_var_screeninfo *var,
			   struct fb_info *info);
static int vfb_mmap(struct fb_info *info, struct file *file,
		    struct vm_area_struct *vma);

static int vfb_open(struct fb_info *info, int user);
static int vfb_release(struct fb_info *info, int user);


static void pcf_fillrect(struct fb_info *p, const struct fb_fillrect *rect);
static void pcf_copyarea(struct fb_info *p, const struct fb_copyarea *area);

static void pcf_imageblit(struct fb_info *p, const struct fb_image *image);

//Michael:060214
//cfb_fillrect & cfb_copyarea & cfb_imageblit will be called by system and clean our framebuffer,
//so we provide three dummy function for vfb_ops.
static struct fb_ops vfb_ops = {
        .fb_check_var           = vfb_check_var,
        .fb_set_par             = vfb_set_par,
        .fb_setcolreg           = vfb_setcolreg,
        .fb_pan_display         = vfb_pan_display,
//	.fb_fillrect            = cfb_fillrect,
//        .fb_copyarea            = cfb_copyarea,
	  .fb_fillrect            = pcf_fillrect,
         .fb_copyarea            = pcf_copyarea,

//        .fb_imageblit           = cfb_imageblit,
        .fb_imageblit           = pcf_imageblit,

        .fb_cursor              = soft_cursor,
        .fb_mmap                = vfb_mmap,

	//Michael
	//.fb_write		= omap_8833fb_write,
	.fb_open=vfb_open,
	.fb_release=vfb_release
	
};

static void pcf_imageblit(struct fb_info *p, const struct fb_image *image)
{
	//printk("pcf_imageblit\n");
}

static void pcf_fillrect(struct fb_info *p, const struct fb_fillrect *rect)
{
	//printk("pcf_fillrect");
}


static void pcf_copyarea(struct fb_info *p, const struct fb_copyarea *area)
{
	//printk("pcf_copyarea\n");
}

    /*
     *  Internal routines
     */



static u_long get_line_length(int xres_virtual, int bpp)
{
	u_long length;

	length = xres_virtual * bpp;
	length = (length + 31) & ~31;
	length >>= 3;
	return (length);
}

    /*
     *  Setting the video mode has been split into two parts.
     *  First part, xxxfb_check_var, must not write anything
     *  to hardware, it should only verify and adjust var.
     *  This means it doesn't alter par but it does use hardware
     *  data from it to check this var. 
     */

static int vfb_check_var(struct fb_var_screeninfo *var,
			 struct fb_info *info)
{
	u_long line_length;

	/*
	 *  FB_VMODE_CONUPDATE and FB_VMODE_SMOOTH_XPAN are equal!
	 *  as FB_VMODE_SMOOTH_XPAN is only used internally
	 */

	if (var->vmode & FB_VMODE_CONUPDATE) {
		var->vmode |= FB_VMODE_YWRAP;
		var->xoffset = info->var.xoffset;
		var->yoffset = info->var.yoffset;
	}

	/*
	 *  Some very basic checks
	 */
	if (!var->xres)
		var->xres = 1;
	if (!var->yres)
		var->yres = 1;
	if (var->xres > var->xres_virtual)
		var->xres_virtual = var->xres;
	if (var->yres > var->yres_virtual)
		var->yres_virtual = var->yres;
	if (var->bits_per_pixel <= 1)
		var->bits_per_pixel = 1;
	else if (var->bits_per_pixel <= 8)
		var->bits_per_pixel = 8;
	else if (var->bits_per_pixel <= 16)
		var->bits_per_pixel = 16;
	else if (var->bits_per_pixel <= 24)
		var->bits_per_pixel = 24;
	else if (var->bits_per_pixel <= 32)
		var->bits_per_pixel = 32;
	else
		return -EINVAL;

	if (var->xres_virtual < var->xoffset + var->xres)
		var->xres_virtual = var->xoffset + var->xres;
	if (var->yres_virtual < var->yoffset + var->yres)
		var->yres_virtual = var->yoffset + var->yres;

	/*
	 *  Memory limit
	 */
	line_length =
	    get_line_length(var->xres_virtual, var->bits_per_pixel);
	if (line_length * var->yres_virtual > videomemorysize)
		return -ENOMEM;

	/*
	 * Now that we checked it we alter var. The reason being is that the video
	 * mode passed in might not work but slight changes to it might make it 
	 * work. This way we let the user know what is acceptable.
	 */
	switch (var->bits_per_pixel) {
	case 1:
	case 8:
		var->red.offset = 0;
		var->red.length = 8;
		var->green.offset = 0;
		var->green.length = 8;
		var->blue.offset = 0;
		var->blue.length = 8;
		var->transp.offset = 0;
		var->transp.length = 0;
		break;
	case 16:		/* RGBA 5551 */
		if (var->transp.length) {
			var->red.offset = 0;
			var->red.length = 5;
			var->green.offset = 5;
			var->green.length = 5;
			var->blue.offset = 10;
			var->blue.length = 5;
			var->transp.offset = 15;
			var->transp.length = 1;
		} else {	/* RGB 565 */
			var->red.offset = 0;
			var->red.length = 5;
			var->green.offset = 5;
			var->green.length = 6;
			var->blue.offset = 11;
			var->blue.length = 5;
			var->transp.offset = 0;
			var->transp.length = 0;
		}
		break;
	case 24:		/* RGB 888 */
		var->red.offset = 0;
		var->red.length = 8;
		var->green.offset = 8;
		var->green.length = 8;
		var->blue.offset = 16;
		var->blue.length = 8;
		var->transp.offset = 0;
		var->transp.length = 0;
		break;
	case 32:		/* RGBA 8888 */
		var->red.offset = 0;
		var->red.length = 8;
		var->green.offset = 8;
		var->green.length = 8;
		var->blue.offset = 16;
		var->blue.length = 8;
		var->transp.offset = 24;
		var->transp.length = 8;
		break;
	}
	var->red.msb_right = 0;
	var->green.msb_right = 0;
	var->blue.msb_right = 0;
	var->transp.msb_right = 0;

	return 0;
}

/* This routine actually sets the video mode. It's in here where we
 * the hardware state info->par and fix which can be affected by the 
 * change in par. For this driver it doesn't do much. 
 */
static int vfb_set_par(struct fb_info *info)
{
	info->fix.line_length = get_line_length(info->var.xres_virtual,
						info->var.bits_per_pixel);
	return 0;
}

    /*
     *  Set a single color register. The values supplied are already
     *  rounded down to the hardware's capabilities (according to the
     *  entries in the var structure). Return != 0 for invalid regno.
     */

static int vfb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
			 u_int transp, struct fb_info *info)
{
	if (regno >= 256)	/* no. of hw registers */
		return 1;
	/*
	 * Program hardware... do anything you want with transp
	 */

	/* grayscale works only partially under directcolor */
	if (info->var.grayscale) {
		/* grayscale = 0.30*R + 0.59*G + 0.11*B */
		red = green = blue =
		    (red * 77 + green * 151 + blue * 28) >> 8;
	}

	/* Directcolor:
	 *   var->{color}.offset contains start of bitfield
	 *   var->{color}.length contains length of bitfield
	 *   {hardwarespecific} contains width of RAMDAC
	 *   cmap[X] is programmed to (X << red.offset) | (X << green.offset) | (X << blue.offset)
	 *   RAMDAC[X] is programmed to (red, green, blue)
	 * 
	 * Pseudocolor:
	 *    uses offset = 0 && length = RAMDAC register width.
	 *    var->{color}.offset is 0
	 *    var->{color}.length contains widht of DAC
	 *    cmap is not used
	 *    RAMDAC[X] is programmed to (red, green, blue)
	 * Truecolor:
	 *    does not use DAC. Usually 3 are present.
	 *    var->{color}.offset contains start of bitfield
	 *    var->{color}.length contains length of bitfield
	 *    cmap is programmed to (red << red.offset) | (green << green.offset) |
	 *                      (blue << blue.offset) | (transp << transp.offset)
	 *    RAMDAC does not exist
	 */
#define CNVT_TOHW(val,width) ((((val)<<(width))+0x7FFF-(val))>>16)
	switch (info->fix.visual) {
	case FB_VISUAL_TRUECOLOR:
	case FB_VISUAL_PSEUDOCOLOR:
		red = CNVT_TOHW(red, info->var.red.length);
		green = CNVT_TOHW(green, info->var.green.length);
		blue = CNVT_TOHW(blue, info->var.blue.length);
		transp = CNVT_TOHW(transp, info->var.transp.length);
		break;
	case FB_VISUAL_DIRECTCOLOR:
		red = CNVT_TOHW(red, 8);	/* expect 8 bit DAC */
		green = CNVT_TOHW(green, 8);
		blue = CNVT_TOHW(blue, 8);
		/* hey, there is bug in transp handling... */
		transp = CNVT_TOHW(transp, 8);
		break;
	}
#undef CNVT_TOHW
	/* Truecolor has hardware independent palette */
	if (info->fix.visual == FB_VISUAL_TRUECOLOR) {
		u32 v;

		if (regno >= 16)
			return 1;

		v = (red << info->var.red.offset) |
		    (green << info->var.green.offset) |
		    (blue << info->var.blue.offset) |
		    (transp << info->var.transp.offset);
		switch (info->var.bits_per_pixel) {
		case 8:
			break;
		case 16:
			((u32 *) (info->pseudo_palette))[regno] = v;
			break;
		case 24:
		case 32:
			((u32 *) (info->pseudo_palette))[regno] = v;
			break;
		}
		return 0;
	}
	return 0;
}

    /*
     *  Pan or Wrap the Display
     *
     *  This call looks only at xoffset, yoffset and the FB_VMODE_YWRAP flag
     */

static int vfb_pan_display(struct fb_var_screeninfo *var,
			   struct fb_info *info)
{
	if (var->vmode & FB_VMODE_YWRAP) {
		if (var->yoffset < 0
		    || var->yoffset >= info->var.yres_virtual
		    || var->xoffset)
			return -EINVAL;
	} else {
		if (var->xoffset + var->xres > info->var.xres_virtual ||
		    var->yoffset + var->yres > info->var.yres_virtual)
			return -EINVAL;
	}
	info->var.xoffset = var->xoffset;
	info->var.yoffset = var->yoffset;
	if (var->vmode & FB_VMODE_YWRAP)
		info->var.vmode |= FB_VMODE_YWRAP;
	else
		info->var.vmode &= ~FB_VMODE_YWRAP;
	return 0;
}

    /*
     *  Most drivers don't need their own mmap function 
     */

static struct page *
vfb_vma_nopage(struct vm_area_struct *vma, unsigned long addr, int *type)
{
	char *sfp;
	struct page *page = NOPAGE_SIGBUS;
	void *page_ptr = NULL;
	unsigned long offset;

	//FN_IN;

	if ((NULL == vma) || (!(sfp = (char *) vma->vm_private_data)))
		return page;

	offset = addr - vma->vm_start;
	if (offset >= videomemorysize)
		return page;

	//DPRINTK( "vfb_vma_nopage offset = %d\n", offset );

	page_ptr =  sfp + offset;
	page = vmalloc_to_page(page_ptr);
	get_page(page);	/* increment page count */

	if (type)
		*type = VM_FAULT_MINOR;

	return page;
}

static struct vm_operations_struct vfb_mmap_vm_ops = {
	.nopage = vfb_vma_nopage,
};

static int vfb_mmap(struct fb_info *info, struct file *file, struct vm_area_struct *vma)
{

	FN_IN;
	unsigned long req_sz;

//	DPRINTK( "vfb_mmap is called\n" );

	if ( (!info) || (!vma) || (!file) )
		return -ENXIO;

	if (vma->vm_pgoff)
		return -EINVAL;	/* want no offset */

	req_sz = vma->vm_end - vma->vm_start;

	if (req_sz > videomemorysize)
		return -ENOMEM;	/* cannot map more than reserved buffer */

	//Alan ?? - checking is needed
//	vma->vm_flags |= (VM_RESERVED | VM_IO);
	vma->vm_flags |= (VM_RESERVED);
	vma->vm_private_data = info_saved->screen_base;
	vma->vm_ops = &vfb_mmap_vm_ops;

//	DPRINTK("mmap %p: %08lx-%08lx pgoff %08lx\n",
//		info_saved->screen_base,vma->vm_start,vma->vm_end,vma->vm_pgoff);

	//Michael
	isWriting=1;
	return 0;
}



static int vfb_open(struct fb_info *info, int user)
{

	FN_IN;
	return 0;
}

static int vfb_release(struct fb_info *info, int user)
{
	FN_IN;
	//Michael:10sec
	//udelay(10000000);
	//udelay(35000);
	//isWriting=0;
	return 0;
}








int __init vfb_setup(char *options)
{

	FN_IN;
	char *this_opt;

	vfb_enable = 1;

	if (!options || !*options)
		return 1;

	while ((this_opt = strsep(&options, ",")) != NULL) {
		if (!*this_opt)
			continue;
		if (!strncmp(this_opt, "disable", 7))
			vfb_enable = 0;
	}
	return 1;
}

    /*
     *  Initialisation
     */

static void vfb_platform_release(struct device *device)
{
	// This is called when the reference count goes to zero.
}

static int __init vfb_probe(struct device *device)
{
	FN_IN;
	struct platform_device *dev = to_platform_device(device);
	struct fb_info *info;
	int retval = -ENOMEM;


	/*
	 * For real video cards we use ioremap.
	 */
	if (!(videomemory = vmalloc(videomemorysize)))
		return retval;

	/*
	 * VFB must clear memory to prevent kernel info
	 * leakage into userspace
	 * VGA-based drivers MUST NOT clear memory if
	 * they want to be able to take over vgacon
	 */
	memset(videomemory, 0, videomemorysize);

	info = framebuffer_alloc(sizeof(u32) * 256, &dev->dev);
	if (!info)
		goto err;

	info->screen_base = videomemory;
	info->fbops = &vfb_ops;

	retval = fb_find_mode(&info->var, info, NULL,
			      NULL, 0, NULL, 8);

	if (!retval || (retval == 4))
		info->var = vfb_default;
	info->fix = vfb_fix;
	info->pseudo_palette = info->par;
	info->par = NULL;
	info->flags = FBINFO_FLAG_DEFAULT;
	retval = fb_alloc_cmap(&info->cmap, 256, 0);
	if (retval < 0)
		goto err1;

        //Alan added
	info_saved = info;

	retval = register_framebuffer(info);

        if (retval < 0)
		goto err2;
	
        dev_set_drvdata(&dev->dev, info);

	DPRINTK(KERN_INFO
	       "fb%d: Virtual frame buffer device, using %ldK of video memory\n",
	       info->node, videomemorysize >> 10);

        //Owen Lin added


       

	sema_init(&gUpdateSem, 0);
	sema_init(&gUpdateSem2, 0);
	gUpdateThreadPid = kernel_thread(lcd_update_thread, 0, CLONE_KERNEL/*0*/);	//ron_lee modify flag to CLONE_KERNEL

       init_timer( &pcf8833_timer );
       pcf8833_timer.expires = PCF8833_TIMER_WUT;
       pcf8833_timer.data = (unsigned long) info_saved;
       pcf8833_timer.function = & omap_8833fb_timer;
       add_timer( & pcf8833_timer );  

        return 0;
err2:
	fb_dealloc_cmap(&info->cmap);
err1:
	framebuffer_release(info);
err:
	vfree(videomemory);
	return retval;
}

static int vfb_remove(struct device *device)
{

	FN_IN;
	struct fb_info *info = dev_get_drvdata(device);
	pcf8833_timer.data = 0;

	if (info) {
		unregister_framebuffer(info);
		vfree(videomemory);
		framebuffer_release(info);
	}
	return 0;
}
#ifdef WIFI_PHONE_LCD
#define NT7523_START_OSC	0x0000
#define NT7523_POWER_CTL1	0x0003
	#define POWER_STB	(1 << 0)
	#define POWER_SLP	(2 << 0)
void set_lcd_nt7523_reg (unsigned short reg, unsigned short val);
unsigned short get_lcd_nt7523_reg (unsigned short reg);

void vfb_shutdown(struct device * dev)
{
	down_interruptible (&gUpdateSem);
	mdelay(100);
	OMAP_PCF8833_CSTN_COMMAND = ((0x0007>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0007; 
	OMAP_PCF8833_CSTN_DATA = ((0x0000>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0000;
	mdelay(20);//20ms

	OMAP_PCF8833_CSTN_COMMAND = ((0x0003>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0003; 
	OMAP_PCF8833_CSTN_DATA = ((0x640C>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x640C;
	mdelay(20);//20ms

	OMAP_PCF8833_CSTN_COMMAND = ((0x0003>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0003; 
	OMAP_PCF8833_CSTN_DATA = ((0x6000>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x6000;
	mdelay(20);//20ms

	OMAP_PCF8833_CSTN_COMMAND = ((0x0002>> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = 0x0002; 
	OMAP_PCF8833_CSTN_DATA = ((0x0400>> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = 0x0400;
	up(&gUpdateSem);
}
static int lcm_pm_callback(struct pm_dev *pmdev, pm_request_t pmreq, void *pmdata)
{
	switch (pmreq) {
		case PM_SUSPEND:
			vfb_suspend(NULL, 0, 0);
			break;
		case PM_RESUME:
			vfb_resume(NULL, 0, 0);
			break;
	}
}
static int vfb_suspend(struct device * dev, u32 state, u32 level)
{
	info("vfb_suspend");
	
	down_interruptible(&gUpdateSem);
	mdelay(100);
	set_lcd_backlight(0);

	/*Do not turn off ic
	OMAP_PCF8833_CSTN_COMMAND = ((0x0003>> 8 ) & 0xff);
       OMAP_PCF8833_CSTN_COMMAND = 0x0003;
       OMAP_PCF8833_CSTN_DATA = ((0x6C05>> 8 ) & 0xff);		//hard code
       OMAP_PCF8833_CSTN_DATA = 0x6C05;
	mdelay(100);
	*/
	return 0;
}

static int vfb_resume(struct device * dev, u32 level)
{
	info("vfb_resume");
	
	/*Do not turn off ic
	OMAP_PCF8833_CSTN_COMMAND = ((0x0003>> 8 ) & 0xff);
       OMAP_PCF8833_CSTN_COMMAND = 0x0003;
       OMAP_PCF8833_CSTN_DATA = ((0x6C04>> 8 ) & 0xff);		//hard code
       OMAP_PCF8833_CSTN_DATA = 0x6C04;
	mdelay(100);
	*/
	//set_lcd_backlight(1); 		//let user mode AP to turn on backlight

	up(&gUpdateSem);
	
	return 0;
}
#endif	//WIFI_PHONE_LCD

static struct device_driver vfb_driver = {
	.name	= "nt7523vfb",
	.bus	= &platform_bus_type,
	.probe	= vfb_probe,
	.remove = vfb_remove,
#ifdef WIFI_PHONE_LCD
	.shutdown = vfb_shutdown,
	.suspend = vfb_suspend,
	.resume  = vfb_resume,
#endif
};

static struct platform_device vfb_device = {
	.name	= "nt7523vfb",
	.id	= 0,
	.dev	= {
		.release = vfb_platform_release,
	}
};

int __init vfb_init(void)
{

	FN_IN;
	int ret = 0;

#ifndef MODULE
	char *option = NULL;

	if (fb_get_options("vfb", &option))
		return -ENODEV;
	vfb_setup(option);
#endif

	if (!vfb_enable)
		return -ENXIO;


	//Michael: Initial the NT7523 chip
	
	if ( omap_8833fb_bus_init() < 0 ) return -ENODEV;

        ret = platform_device_register(&vfb_device);
        DPRINTK( "platform_device_register: %d\n", ret );

	if (!ret) {
        	ret = driver_register(&vfb_driver);
                DPRINTK( "driver_register: %d\n", ret );
		if (ret)
			driver_unregister(&vfb_driver);
	}

#ifdef CONFIG_PM
  	if(!(lcm_pm_dev = pm_register(PM_UNKNOWN_DEV, 76, lcm_pm_callback)))	//register the LCM id = 76 'L'
		printk("Failed to register PM callback\n");
#endif

	return ret;
}

#ifdef LCD_DMA

/**/
static __inline__ void lcd_dma_callback(int lch,u16 ch_status, void* data)
{
	//FN_IN;
#ifdef LCD_DMA_TEST
	//DPRINTK(KERN_ALERT "Michael:dmabuf_tx=%u ; dmabuf_tx2=%u\n",*dmabuf_tx, *dmabuf_tx2);
	int index = 0;
		for( index = 0; index < LCD_PIXEL_COUNT; ++ index ) 
		{
			// Assume the color format is RGB565
			__u16 data = dmabuf_tx2[ index ];
              	//__u16 data = *dmabuf_tx;	
			//dmabuf_tx++;
			OMAP_PCF8833_CSTN_DATA = (( data >> 8 ) );
			OMAP_PCF8833_CSTN_DATA = (( data));

		}

#endif
	up( &gUpdateSem2 );
}

#endif

/****************************************************************************
*  Michael:
*  lcd_update_thread
*
*  Worker thread to transfer data to the LCD.
*
***************************************************************************/
static int lcd_update_thread(void *data)
{

 int kthreadcount = 0;
 	current->flags |= PF_NOFREEZE;
    /* Run until signal received */
    while (1)
    {
 //   	return 0; omap_readl(a)
	//DPRINTK(KERN_ALERT "Michael:lcd_update_thread\n");
	if ( down_interruptible (&gUpdateSem) == 0 )
        {
	


		//DPRINTK(KERN_ALERT "Michael:lcd_update_thread::down_interruptible\n");

	        __u16* src = (__u16*) info_saved->screen_base;        
        	int width  = info_saved->var.xres;
        	int height = info_saved->var.yres;

		omap_8833fb_setX( 0, width-1 );
		omap_8833fb_setY( 0, height-1 );

		//DPRINTK(KERN_ALERT "setX=%d, setY=%d\n", width-1, height-1);
        
		int index = 0, bound = (width * height); 

		OMAP_PCF8833_CSTN_COMMAND = ((0x0022>> 8)& 0xff);
		OMAP_PCF8833_CSTN_COMMAND = 0x0022;


		
#ifdef LCD_DMA

#ifdef LCD_DMA_TEST

	//*dmabuf_tx = kthreadcount;
	memcpy(dmabuf_tx, src, OMAP_PCF8833_CSTN__SIZE);

	omap_set_dma_src_params(dma_lch_tx,
				OMAP_DMA_PORT_EMIFF,
				OMAP_DMA_AMODE_POST_INC,
				 dmaphys_tx) ;
	omap_set_dma_transfer_params(dma_lch_tx,
				     OMAP_DMA_DATA_TYPE_S8,
				     LCD_PIXEL_COUNT * 2, 1,
				     OMAP_DMA_SYNC_ELEMENT); 

#else

		memcpy(dmabuf_tx, src, OMAP_PCF8833_CSTN__SIZE);

/**/
		omap_set_dma_src_params(dma_lch_tx,
				OMAP_DMA_PORT_EMIFF,
				OMAP_DMA_AMODE_POST_INC,
				 dmaphys_tx+1) ;

		omap_set_dma_src_index(dma_lch_tx, -1,2);

		omap_set_dma_transfer_params(dma_lch_tx,
				     OMAP_DMA_DATA_TYPE_S8,
				       2, LCD_PIXEL_COUNT,
				     OMAP_DMA_SYNC_FRAME); 

#endif


		omap_start_dma(dma_lch_tx) ;

		down_interruptible (&gUpdateSem2);

/*
		//Make sure that data in dmabuf_tx is correct
		for( index = 0; index < bound; ++ index ) 
		{
			// Assume the color format is RGB565
			__u16 data = dmabuf_tx[ index ];
              	//__u16 data = *dmabuf_tx;	
			//dmabuf_tx++;
			OMAP_PCF8833_CSTN_DATA = (( data >> 8 ) & 0xff);
			OMAP_PCF8833_CSTN_DATA = (( data) & 0xff);

		}
*/

#else

		//DPRINTK(KERN_ALERT "Michael:LCD_KernelThread Trigger\n");
		//int ccount = COLORCOUNT % 5; 
		//++COLORCOUNT;


		for( index = 0; index < bound; ++ index ) 
		{
			

			//__u16 data = src[ index ];	
			//Use pointer to replace ArrayOP for speed reason
			//Because we don't know if compiler will optimize ArrayOP for us
			__u16 data = *src;
			src++;
			
			// Assume the color format is RGB565
			//__u16 data = COLORARRAY[ccount];
		
			OMAP_PCF8833_CSTN_DATA = (( data >> 8 ) & 0xff);
			OMAP_PCF8833_CSTN_DATA = (( data) & 0xff);
		}

#endif


		kthreadcount++;
        }
        else
            break;
    }

	
} // lcd_update_thread
#define WIFI_PHONE_LCD
#ifdef WIFI_PHONE_LCD
/* added by Tommy */
void set_lcd_nt7523_reg (unsigned short reg, unsigned short val)
{	
	DPRINTK("[set_lcd_nt7523_reg]-reg:0x%04x, val:0x%04x\n", reg, val);
	down_interruptible (&gUpdateSem);
	mdelay(100);
	OMAP_PCF8833_CSTN_COMMAND = ((reg >> 8 ) & 0xff); 
	OMAP_PCF8833_CSTN_COMMAND = reg;
	OMAP_PCF8833_CSTN_DATA = ((val >> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_DATA = val & 0xff;
	mdelay(100);
	up(&gUpdateSem);
}

//get LCM register seems not work
unsigned short get_lcd_nt7523_reg (unsigned short reg)
{	
	unsigned short ret;
	down_interruptible(&gUpdateSem);
	DPRINTK("[get_lcd_nt7523_reg]-reg:0x%04x\n", reg);
	OMAP_PCF8833_CSTN_COMMAND = ((reg >> 8 ) & 0xff);
	OMAP_PCF8833_CSTN_COMMAND = reg;
	ret = OMAP_PCF8833_CSTN_DATA;
	ret = (ret << 8) | OMAP_PCF8833_CSTN_DATA;
	up(&gUpdateSem);
}
EXPORT_SYMBOL(get_lcd_nt7523_reg);
EXPORT_SYMBOL(set_lcd_nt7523_reg);
#endif	//WIFI_PHONE_LCD

module_init(vfb_init);

#ifdef MODULE
static void __exit vfb_exit(void)
{
        omap_free_gpio( 49 );     
	platform_device_unregister(&vfb_device);
	driver_unregister(&vfb_driver);
}

module_exit(vfb_exit);

MODULE_LICENSE("GPL");
#endif				/* MODULE */
