/* 
 * linux/include/asm-arm/arch-omap2/io.h
 * 
 * IO definitions for TI OMAP2 processors and boards
 *
 * Copyright (C) 2004 Texas Instruments, Inc. 
 *
 * Copied from linux/include/asm-arm/arch-sa1100/io.h:
 *   Copyright (C) 1997-1999 Russell King
 *
 * This package is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. 
 * 
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED 
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
 */

#ifndef __ASM_ARM_ARCH_OMAP2_IO_H
#define __ASM_ARM_ARCH_OMAP2_IO_H

#include <asm/sizes.h>
#include <asm/types.h>

#define IO_SPACE_LIMIT        0xffffffff

/* there are no PCI buses ...but a lot of drivers depend on this */
#define PCIO_BASE             0
#define __io(a)               ((a))
#ifndef __ASSEMBLER__
#define __mem_pci(a)          ((unsigned long)(a))
#define __mem_isa(a)          (PCIO_BASE + (unsigned long)(a))
#endif

/*
 * Virtual base address where OMAP2 IO regions are remapped to.
 *  These devices reside on the L3 and L4 bus's.
 *  L3:vio(680xxxxx) ->F80xxxxx, L4:vio(480xxxxx)->0xd8000000
 */
#define IO_L3L4_MASK          0x90000000
#define IO_VIRT_L3            0xF8000000
#define IO_SIZE_L3            SZ_1M
#define IO_VIRT_L4            0xD8000000
#define IO_SIZE_L4            SZ_1M
#define IO_ADDRESS(pa)        ((pa)|IO_L3L4_MASK)

#define io_p2v(pa)            ((pa)|IO_L3L4_MASK)
#define io_v2p(va)            ((va)& ~IO_L3L4_MASK)

#ifndef __ASSEMBLER__
/*
 * Functions to access the OMAP IO region
 *
 * NOTE: - Use omap_read/write[bwl] for physical register addresses
 *     - Use __raw_read/write[bwl]() for virtual register addresses
 *     - Use IO_ADDRESS(phys_addr) to convert registers to virtual addresses
 *     - DO NOT use hardcoded virtual addresses to allow changing the
 *       IO address space again if needed
 */
#define omap_readb(a)         (*(volatile unsigned char  *)IO_ADDRESS(a))
#define omap_readw(a)         (*(volatile unsigned short *)IO_ADDRESS(a))
#define omap_readl(a)         (*(volatile unsigned int   *)IO_ADDRESS(a))

#define omap_writeb(v,a)      (*(volatile unsigned char  *)IO_ADDRESS(a) = (v))
#define omap_writew(v,a)      (*(volatile unsigned short *)IO_ADDRESS(a) = (v))
#define omap_writel(v,a)      (*(volatile unsigned int   *)IO_ADDRESS(a) = (v))

/* 16 bit uses LDRH/STRH, base +/- offset_8 */
typedef struct {
	volatile u16 offset[256];
} __regbase16;
#define __REGV16(vaddr)       ((__regbase16 *)((vaddr)&~0xff)) \
                                  ->offset[((vaddr)&0xff)>>1]
#define __REG16(paddr)        __REGV16(io_p2v(paddr))

/* 8/32 bit uses LDR/STR, base +/- offset_12 */
typedef struct {
	volatile u8 offset[4096];
} __regbase8;
#define __REGV8(vaddr)        ((__regbase8  *)((vaddr)&~4095)) \
                                  ->offset[((vaddr)&4095)>>0]
#define __REG8(paddr)         __REGV8(io_p2v(paddr))

typedef struct {
	volatile u32 offset[4096];
} __regbase32;
#define __REGV32(vaddr)       ((__regbase32 *)((vaddr)&~4095)) \
                                  ->offset[((vaddr)&4095)>>2]
#define __REG32(paddr)        __REGV32(io_p2v(paddr))

#else

#define __REG8(paddr)         io_p2v(paddr)
#define __REG16(paddr)        io_p2v(paddr)
#define __REG32(paddr)        io_p2v(paddr)

#endif				/* __ASSEMBLER__ */

#endif				/* __ASM_ARM_ARCH_OMAP2_IO_H */
