/**
*  @file   kmsg.h
*  @brief  This module defines the functions for error message or 
*          debug message or information. All applications should
*          use these functions to show message.
*
*  Copyright (C)
*       2006 Accton Technology Corporation
*
*  @author Sean Hsieh (sean_hsieh@accton.com.tw)
*  @author Michael Ruan (michael_ruan@accton.com.tw)
*
*  References:
*    if needed ( code referenced, data book, ...)
*
*  History:
*    02/06/2006 Sean Hsieh,  define whole mechanism and code
*    02/06/2006 Michael Ruan, modify for kernel use
*
**/
#ifndef _PMSG_H_
#define _PMSG_H_

/*** Include file section
 ***      each include file shall self-contain, does not include other header file.
 ***/

/*** Constant definition section
 ***      define the constants and structures used for DCL_SYS module.
 ***/

/*** Structure / type definition section
 ***      define the structures used for all DCL modules
 ***/
typedef enum KMSG_MSG_LEVEL_E
{
    KMSG_MSG_LEVEL_FATAL = 1,
    KMSG_MSG_LEVEL_CRITICAL,
    KMSG_MSG_LEVEL_ERROR,
    KMSG_MSG_LEVEL_SYSTEM,
    KMSG_MSG_LEVEL_DEBUG,
    KMSG_MSG_LEVEL_WARNING,
    KMSG_MSG_LEVEL_INFOMATION
}   KMSG_MSG_LEVEL_T;
/*** Export functions section
 ***      define the export functions for applications
 ***/
/**
 **  PMSG_printMsg() and PMSG_printErr() SHALL NOT be called by any caller,
 **  it's used for redirect function.
 **/
extern void KMSG_printMsg( int buglevel, const char *prefix, const char *file, const char *func, int line, const char *fmt, ... );

/**   Name:     PRINT_MSG
 **   Function: Show the message to output (stdout)
 **   Input:
 **   @param    buglevel - the buglevel control which level of debug message should
 **                        be print out.
 **             prefix   - the level of message; refer PMSG_MSG_LEVEL_T.
 **                        default display-level is SYSTEM, ie. for the msg level
 **                        great than SYSTEM, won't be shown.
 **   @param    fmt      - the format and parameters for output
 **   Output:
 **   @return  NONE
 **   Notes:
 **     
 **/
#define PRINT_MSG( buglevel, prefix, fmt... ) \
    KMSG_printMsg( buglevel, prefix, __FILE__, __FUNCTION__, __LINE__, ##fmt );


/**   Name:     __KMSG_setDebugLevel
 **			  __KMSG_getDebugLevel
 **   Function: set the display threshold value, default is KMSG_MSG_LEVEL_CRITICAL
 **   Input:
 **   @param    level - the level of message; refer PMSG_MSG_LEVEL_T.
 **                     default display-level is KMSG_MSG_LEVEL_CRITICAL, ie. for the msg level
 **                     great than it, won't be shown.

 **   Notes:
 **      1. Those function should be called through /proc/kdbmsg.
 **      2. For normal release, it uses default setting.
 **/
extern void __KMSG_setDebugLevel( int level );
extern int __KMSG_getDebugLevel(void);

/**   Name:     KMSG_setOutputFields
 **   Function: enable/disable the output field, default is all enable
 **   Input:
 **   @param    0 - disable / 1 -enable for 6 fields
 **
 **   @return  NONE
 **   Notes:
 **     
 **      
 **/
extern void KMSG_setOutputFields( int prefix,  int time, int file, int func, int line, int msg );

#endif  /* end of PMSG.h */
